# Copyright (C) 2021-2024 Analog Devices, Inc.
#
# SPDX short identifier: ADIBSD

from adi.attribute import attribute
from adi.context_manager import context_manager
from adi.rx_tx import rx


class ad7799(rx, context_manager):
    """ AD7799 ADC """

    _complex_data = False
    channel = []  # type: ignore
    _rx_channel_names = ["channel0", "channel1", "channel2"]
    _device_name = ""

    def __init__(self, uri=""):

        context_manager.__init__(self, uri, self._device_name)

        self._rxadc = self._ctx.find_device("AD7799")

        self.channel = []
        for name in self._rx_channel_names:
            self.channel.append(self._channel(self._rxadc, name))

    @property
    def gain(self):
        """Get gain of the AD7799"""
        return self._get_iio_dev_attr_str("gain", self._rxadc)

    @gain.setter
    def gain(self, value):
        """Sets gain of the AD7799"""
        self._set_iio_dev_attr_str("gain", value, self._rxadc)

    class _channel(attribute):
        """AD7799 channel"""

        def __init__(self, ctrl, channel_name):
            self.name = channel_name
            self._ctrl = ctrl

        @property
        def value(self):
            """AD7124 channel mV value"""
            return self._get_iio_attr(self.name, "volts", False, self._ctrl)



# Copyright (C) 2022-2024 Analog Devices, Inc.
#
# SPDX short identifier: ADIBSD

import numpy as np
from adi.context_manager import context_manager
from adi.rx_tx import rx


class ltc2387(rx, context_manager):

    """LTC2387 family devices"""

    _device_name = ""
    _rx_data_type = np.int32
    _complex_data = False
    _rx_channel_names = ["voltage"]

    def __init__(self, uri="", device_index=0):

        context_manager.__init__(self, uri, self._device_name)

        compatible_parts = [
            "ltc2387",
            "ltc2387-16",
            "ltc2387-18",
            "adaq23876",
            "adaq23878",
        ]

        self._ctrl = None
        index = 0

        # Selecting the device_index-th device from the xxx2387 family as working device.
        for device in self._ctx.devices:
            if device.name in compatible_parts:
                if index == device_index:
                    self._ctrl = device
                    self._rxadc = device
                    break
                else:
                    index += 1

        rx.__init__(self)

    @property
    def sampling_frequency(self):

        """sample_rate: Sample rate in samples per second.
        Valid options are:
        Device's maximum sample rate (15000000 in the case of the LTC2387-18) and lower.
        Actual sample rates will be the master clock divided by an integer, for example,
        the CN0577 has a 120 MHz clock, so available sample rates will be:
        120 MHz / 8 = 15 Msps
        120 MHz / 9 = 13.333 Msps
        120 MHz / 10 = 12 Msps
        etc.
        """
        return self._get_iio_dev_attr("sampling_frequency")

    @sampling_frequency.setter
    def sampling_frequency(self, value):
        self._set_iio_dev_attr_str("sampling_frequency", value)



# Copyright (C) 2022-2024 Analog Devices, Inc.
#
# SPDX short identifier: ADIBSD


from adi.attribute import attribute


class sync_start(attribute):
    """ Synchronization Control: This class allows for synchronous transfers
        between transmit and receive data movement or captures.
    """

    @property
    def tx_sync_start(self):
        """ tx_sync_start: Issue a synchronisation request

        Possible values are:
            - **arm**: Writing this key will arm the trigger mechanism sensitive to an
              external sync signal. Once the external sync signal goes high
              it synchronizes channels within a DAC, and across multiple
              instances. This bit has an effect only the EXT_SYNC
              synthesis parameter is set.
            - **disarm**: Writing this key will disarm the trigger mechanism
              sensitive to an external sync signal. This bit has an
              effect only the EXT_SYNC synthesis parameter is set.
            - **trigger_manual**: Writing this key will issue an external sync event
              if it is hooked up inside the fabric. This key has an effect
              only the EXT_SYNC synthesis parameter is set.
              This key self clears.
        """
        try:
            return self._get_iio_dev_attr_str("sync_start_enable", _ctrl=self._txdac)
        except:  # noqa: E722
            if self._txdac.reg_read(0x80000068) & 0x1 == 0x1:
                return "arm"
            else:
                return "disarm"

    @tx_sync_start.setter
    def tx_sync_start(self, value):
        try:
            self._set_iio_dev_attr_str("sync_start_enable", value, _ctrl=self._txdac)
        except:  # noqa: E722
            chan = self._txdac.find_channel("altvoltage0", True)
            chan.attrs["raw"].value = "1"

    @property
    def tx_sync_start_available(self):
        """ tx_sync_start_available: Returns a list of possible keys used for tx_sync_start """
        try:
            return self._get_iio_dev_attr_str(
                "sync_start_enable_available", _ctrl=self._txdac
            )
        except:  # noqa: E722
            return "arm"

    @property
    def rx_sync_start(self):
        """ rx_sync_start: Issue a synchronisation request

        Possible values are:
            - **arm**: Writing this key will arm the trigger mechanism sensitive to an
              external sync signal. Once the external sync signal goes high
              it synchronizes channels within a ADC, and across multiple
              instances. This bit has an effect only the EXT_SYNC
              synthesis parameter is set.
            - **disarm**: Writing this key will disarm the trigger mechanism
              sensitive to an external sync signal. This bit has an
              effect only the EXT_SYNC synthesis parameter is set.
            - **trigger_manual**: Writing this key will issue an external sync event
              if it is hooked up inside the fabric. This key has an effect
              only the EXT_SYNC synthesis parameter is set.
              This key self clears.
        """
        try:
            return self._get_iio_dev_attr_str("sync_start_enable", _ctrl=self._rxadc)
        except:  # noqa: E722
            if self._rxadc.reg_read(0x80000068) & 1 == 1:
                return "arm"
            else:
                return "disarm"

    @rx_sync_start.setter
    def rx_sync_start(self, value):
        try:
            self._set_iio_dev_attr_str("sync_start_enable", value, _ctrl=self._rxadc)
        except:  # noqa: E722
            self._rxadc.reg_write(0x80000044, 0x8)
            while self._rxadc.reg_read(0x80000068) == 0:
                self._rxadc.reg_write(0x80000044, 0x8)

    @property
    def rx_sync_start_available(self):
        """ rx_sync_start_available: Returns a list of possible keys used for rx_sync_start """
        try:
            return self._get_iio_dev_attr_str(
                "sync_start_enable_available", _ctrl=self._rxadc
            )
        except:  # noqa: E722
            return "arm"



# Copyright (C) 2019-2024 Analog Devices, Inc.
#
# SPDX short identifier: ADIBSD

from decimal import Decimal

from adi.context_manager import context_manager
from adi.jesd import jesd
from adi.rx_tx import rx
from adi.sync_start import sync_start


class ad9625(rx, context_manager, sync_start):

    """ AD9625 High-Speed ADC """

    _complex_data = False
    _rx_channel_names = ["voltage0"]
    _device_name = ""

    def __init__(self, uri="", username="root", password="analog"):

        context_manager.__init__(self, uri, self._device_name)

        self._jesd = jesd(uri, username=username, password=password)
        self._rxadc = self._ctx.find_device("axi-ad9625-hpc")

        rx.__init__(self)

    @property
    def rx_sample_rate(self):
        """rx_sample_rate: Sample rate RX path in samples per second"""
        return self._get_iio_attr("voltage0", "sampling_frequency", False, self._rxadc)

    @property
    def scale(self):
        """scale: AD9625 Gain"""
        return float(self._get_iio_attr("voltage0", "scale", False))

    @scale.setter
    def scale(self, value):
        self._set_iio_attr("voltage0", "scale", False, str(Decimal(value).real))

    @property
    def scale_available(self):
        """Provides all available scale settings for the AD9625"""
        return self._get_iio_attr("voltage0", "scale_available", False)

    @property
    def test_mode(self):
        """test_mode: Select Test Mode. Options are:
        off, midscale_short, pos_fullscale, neg_fullscale, checkerboard,
        pn_long, pn_short, one_zero_toggle, user, ramp"""
        return self._get_iio_attr("voltage0", "test_mode", False)

    @test_mode.setter
    def test_mode(self, value):
        self._set_iio_attr("voltage0", "test_mode", False, value, self._rxadc)

    @property
    def jesd204_statuses(self):
        return self._jesd.get_all_statuses()



# Copyright (C) 2020-2024 Analog Devices, Inc.
#
# SPDX short identifier: ADIBSD


from adi.rx_tx import rx, tx


class _property_remap:
    def __init__(self, attr, dev):
        self._dev = dev
        self._attr = attr

    def __get__(self, instance, owner):
        return getattr(self._dev, self._attr)

    def __set__(self, instance, value):
        setattr(self._dev, self._attr, value)


def remap(object_source, old, new, classname):
    """ Remap methods to new names to different object """

    method_list = [
        func
        for func in dir(object_source)
        if not func.startswith("__") and not func.startswith("_")
    ]
    for method in method_list:
        if old not in method:
            continue
        new_method = method.replace(old, new)
        setattr(
            classname, new_method, _property_remap(method, object_source),
        )


class obs(rx):
    """ Buffer handling for observation devices """

    def __init__(self, ctx, obs_dev, channel_names, complex_data=True):
        self._ctx = ctx
        self._rxadc = obs_dev
        self._rx_channel_names = channel_names
        self._rx_complex_data = complex_data
        rx.__init__(self)

    def __del__(self):
        rx.__del__(self)


class tx_two(tx):
    """ Buffer Handling for secondary transmit channels """

    def __init__(self, ctx, obs_dev, channel_names, complex_data=True):
        self._ctx = ctx
        self._txdac = obs_dev
        self._tx_channel_names = channel_names
        self._complex_data = complex_data
        tx.__init__(self)

    def __del__(self):
        tx.__del__(self)



# Copyright (C) 2021-2024 Analog Devices, Inc.
#
# SPDX short identifier: ADIBSD

from adi.attribute import attribute
from adi.context_manager import context_manager
from adi.rx_tx import rx


class adpd1080(rx, context_manager):
    """ADPD1080 photo-electronic device."""

    channel = []  # type: ignore
    _device_name = ""

    def __init__(self, uri="", device_index=0):
        """ADPD1080 class constructor."""
        context_manager.__init__(self, uri, self._device_name)

        compatible_parts = ["adpd1080"]

        self._ctrl = None
        index = 0

        # Selecting the device_index-th device from the adpd188 family as working device.
        for device in self._ctx.devices:
            if device.name in compatible_parts:
                if index == device_index:
                    self._ctrl = device
                    self._rxadc = device
                    break
                else:
                    index += 1

        # dynamically get channels and sorting them after the color
        # self._ctrl.channels.sort(key=lambda x: str(x.id[14:]))
        self._rx_channel_names = []
        self.channel = []
        for ch in self._ctrl._channels:
            name = ch._id
            self._rx_channel_names.append(name)
            self.channel.append(self._channel(self._ctrl, name))
        rx.__init__(self)
        self.rx_buffer_size = 16

    def rx(self):
        buff = super().rx()
        del self._rx__rxbuf
        return buff

    @property
    def sample_rate(self):
        """Sets sampling frequency of the ADPD1080."""
        return self._get_iio_dev_attr_str("sampling_frequency", self._rxadc)

    @sample_rate.setter
    def sample_rate(self, value):
        self._set_iio_dev_attr_str("sampling_frequency", value, self._rxadc)

    class _channel(attribute):
        """ADPD1080 channel."""

        def __init__(self, ctrl, channel_name):
            self.name = channel_name
            self._ctrl = ctrl

        @property
        def raw(self):
            """ADPD1080 channel raw value."""
            return self._get_iio_attr(self.name, "raw", False)

        @property
        def offset(self):
            """ADPD1080 channel offset."""
            return self._get_iio_attr(self.name, "offset", False)

        @offset.setter
        def offset(self, value):
            self._set_iio_attr(self.name, "offset", False, value)



# Copyright (C) 2022-2024 Analog Devices, Inc.
#
# SPDX short identifier: ADIBSD


from decimal import Decimal

import numpy as np
from adi.attribute import attribute
from adi.context_manager import context_manager
from adi.rx_tx import rx


class ad719x(rx, context_manager):

    """ AD719x ADC """

    _complex_data = False
    channel = []  # type: ignore
    _device_name = ""

    def __init__(self, uri="", device_name=""):
        """Constructor for AD719x class."""
        context_manager.__init__(self, uri, self._device_name)

        compatible_parts = ["ad7190", "ad7192", "ad7193", "ad7194", "ad7195"]

        self._ctrl = None

        if not device_name:
            device_name = compatible_parts[0]
        else:
            if device_name not in compatible_parts:
                raise Exception("Not a compatible device: " + device_name)

        # Select the device matching device_name as working device
        for device in self._ctx.devices:
            if device.name == device_name:
                self._ctrl = device
                self._rxadc = device
                break

        if not self._ctrl:
            raise Exception("Error in selecting matching device")

        if not self._rxadc:
            raise Exception("Error in selecting matching device")

        self._rx_channel_names = []
        self.channel = []
        for ch in self._ctrl.channels:
            name = ch._id
            self._rx_channel_names.append(name)
            self.channel.append(self._channel(self._ctrl, name))

        rx.__init__(self)

    class _channel(attribute):

        """ AD719x channel """

        def __init__(self, ctrl, channel_name):
            self.name = channel_name
            self._ctrl = ctrl

        @property
        def raw(self):
            """AD719X channel raw value."""
            return self._get_iio_attr(self.name, "raw", False)

        @property
        def scale(self):
            """AD719X channel scale."""
            return float(self._get_iio_attr_str(self.name, "scale", False))

        @scale.setter
        def scale(self, value):
            self._set_iio_attr(self.name, "scale", False, str(Decimal(value).real))

        @property
        def offset(self):
            """AD719x channel offset."""
            return float(self._get_iio_attr_str(self.name, "offset", False))

        @offset.setter
        def offset(self, value):
            self._set_iio_attr(self.name, "offset", False, str(Decimal(value).real))

    def to_volts(self, index, val):
        """Converts raw value to SI."""
        _scale = self.channel[index].scale

        ret = None

        if isinstance(val, np.int32):
            ret = val * _scale

        if isinstance(val, np.ndarray):
            ret = [x * _scale for x in val]

        if ret is None:
            raise Exception("Error in converting to actual voltage")

        return ret



# Copyright (C) 2022-2024 Analog Devices, Inc.
#
# SPDX short identifier: ADIBSD

import numpy as np
from adi.context_manager import context_manager
from adi.rx_tx import rx


class adaq8092(rx, context_manager):

    """ADAQ8092 14-Bit, 105MSPS, Dual-Channel uModule Data Acquisition Solution"""

    _device_name = "adaq8092"
    _rx_stack_interleaved = True
    _rx_data_type = np.int16

    def __init__(
        self, uri="",
    ):
        """Initialize."""
        context_manager.__init__(self, uri, self._device_name)

        self._ctrl = self._ctx.find_device("adaq8092")
        self._rxadc = self._ctx.find_device("adaq8092")
        self._device_name = "adaq8092"
        if not self._rxadc:
            self._ctrl = self._ctx.find_device("cf_axi_adc")
            self._rxadc = self._ctx.find_device("cf_axi_adc")
            self._device_name = "cf_axi_adc"

        self._rx_channel_names = []
        for ch in self._rxadc.channels:
            name = ch._id
            self._rx_channel_names.append(name)
        rx.__init__(self)

    @property
    def sampling_frequency(self):
        """Get Sampling Frequency."""
        return self._get_iio_dev_attr("sampling_frequency")



# Copyright (C) 2019-2024 Analog Devices, Inc.
#
# SPDX short identifier: ADIBSD

from adi.ad9250 import ad9250
from adi.context_manager import context_manager
from adi.jesd import jesd
from adi.rx_tx import rx


class fmcjesdadc1(ad9250):

    """FMCJESDADC1 Four-Channel High Speed Data Acquisition FMC Board"""

    _split_cores = True
    _rx_channel_names = ["voltage0", "voltage1"]  # Recheck RX Channel Names
    _device_name = ""

    def __init__(self, uri="", username="root", password="analog"):
        context_manager.__init__(self, uri, self._device_name)

        self._jesd = jesd(uri, username=username, password=password)
        self._rxadc = self._ctx.find_device("axi-ad9250-hpc-0")
        self._rxadc_chip_b = self._ctx.find_device("axi-ad9250-hpc-1")

        rx.__init__(self)

    @property
    def test_mode_chan0(self):
        """test_mode_chan0: Select Test Mode. Options are:
        off midscale_short pos_fullscale neg_fullscale checkerboard pn_long pn_short one_zero_toggle user ramp"""
        return self._get_iio_attr("voltage0", "test_mode", False)

    @test_mode_chan0.setter
    def test_mode_chan0(self, value):
        self._set_iio_attr("voltage0", "test_mode", False, value, self._rxadc)

    @property
    def test_mode_chan1(self):
        """test_mode_chan1: Select Test Mode. Options are:
        off midscale_short pos_fullscale neg_fullscale checkerboard pn_long pn_short one_zero_toggle user ramp"""
        return self._get_iio_attr("voltage1", "test_mode", False)

    @test_mode_chan1.setter
    def test_mode_chan1(self, value):
        self._set_iio_attr("voltage1", "test_mode", False, value, self._rxadc)

    @property
    def jesd204_statuses(self):
        return self._jesd.get_all_statuses()



# Copyright (C) 2021-2024 Analog Devices, Inc.
#
# SPDX short identifier: ADIBSD

from adi.attribute import attribute
from adi.context_manager import context_manager


class adf5610(attribute, context_manager):
    """ADF5610 Microwave Wideband Synthesizer with Integrated VCO

    parameters:
        uri: type=string
            URI of IIO context with ADF5610
    """

    _device_name = "adf5610"

    def __init__(self, uri="", device_name=""):

        context_manager.__init__(self, uri, self._device_name)

        self._device_name = device_name
        # Find the device
        self._ctrl = self._ctx.find_device(self._device_name)

        # Raise an exception if the device isn't found
        if not self._ctrl:
            raise Exception("ADF5610 device not found")

    @property
    def frequency(self):
        """Get/Set the Frequency in Hz"""
        return self._get_iio_attr("altvoltage0", "frequency", True, self._ctrl)

    @frequency.setter
    def frequency(self, value):
        """Get/Set the Frequency in Hz"""
        self._set_iio_attr("altvoltage0", "frequency", True, int(value), self._ctrl)

    def reg_read(self, reg):
        """Direct Register Access via debugfs"""
        self._set_iio_debug_attr_str("direct_reg_access", reg, self._ctrl)
        return self._get_iio_debug_attr_str("direct_reg_access", self._ctrl)

    def reg_write(self, reg, value):
        """Direct Register Access via debugfs"""
        self._set_iio_debug_attr_str("direct_reg_access", f"{reg} {value}", self._ctrl)



# Copyright (C) 2022-2024 Analog Devices, Inc.
#
# SPDX short identifier: ADIBSD

from decimal import Decimal

import numpy as np
from adi.attribute import attribute
from adi.context_manager import context_manager
from adi.rx_tx import rx


def _sign_extend(value, nbits):
    sign_bit = 1 << (nbits - 1)
    return (value & (sign_bit - 1)) - (value & sign_bit)


def _bitmask(nbits):
    mask = 0
    for i in range(0, nbits):
        mask = (mask << 1) | 1
    return mask


class ad4630(rx, context_manager, attribute):

    """ AD4630 is low power 24-bit precision SAR ADC """

    _compatible_parts = ["ad4630-24", "ad4030-24", "ad4630-16"]
    _complex_data = False
    _data_type = np.uint32
    _device_name = ""
    _rx_channel_names = []

    """ Default part to initialize is ad4630-24. If you don't hardware test fails"""

    def __init__(self, uri="", device_name="ad4630-24"):

        context_manager.__init__(self, uri, self._device_name)

        if device_name not in self._compatible_parts:
            raise Exception(
                "Not a compatible device: "
                + str(device_name)
                + ". Please select from "
                + str(self.self._compatible_parts)
            )
        else:
            self._ctrl = self._ctx.find_device(device_name)
            self._rxadc = self._ctx.find_device(device_name)

        _channels = []
        self.output_bits = []
        for ch in self._ctrl.channels:
            self.output_bits.append(ch.data_format.bits)
            self._rx_channel_names.append(ch.id)
            _channels.append((ch.id, self._channel(self._ctrl, ch.id)))
            if "0" in ch.id:
                self.chan0 = self._channel(self._ctrl, ch.id)
            if "1" in ch.id:
                self.chan1 = self._channel(self._ctrl, ch.id)

        rx.__init__(self)

    def rx(self):
        data = self._rx_buffered_data()
        temp = []
        if self._num_rx_channels != 2:
            for ch in range(0, self._num_rx_channels):
                nbits = self._ctrl.channels[ch].data_format.bits
                shift = self._ctrl.channels[ch].data_format.shift
                ch_data = np.zeros(data[int(ch / 2)].shape, dtype=np.uint32)
                for index in range(0, len(data[int(ch / 2)])):
                    ch_data[index] = (data[int(ch / 2)][index] >> shift) & _bitmask(
                        nbits
                    )
                temp.append(np.vectorize(_sign_extend)(ch_data, nbits))
            data = temp
        else:
            for idx, ch_data in enumerate(data):
                nbits = self._ctrl.channels[idx].data_format.bits
                temp.append(np.vectorize(_sign_extend)(ch_data, nbits))
            data = np.vectorize(_sign_extend)(data, nbits)

        return data

    def output_data_mode(self):
        """Determine the output data mode in which device is configured."""
        if self.output_bits[0] == 30:
            return "30bit_avg"
        if self.output_bits[0] == 32:
            return "32bit_test_pattern"
        if self.output_bits[0] == 16:
            return "16bit_diff_8bit_cm"
        if len(self.output_bits) == 1 and self.output_bits[0] == 24:
            return "24bit_diff"
        if len(self.output_bits) == 2 and self.output_bits[0] == self.output_bits[1]:
            return "24bit_diff"
        else:
            return "24bit_diff_8bit_cm"

    @property
    def sample_rate(self):
        """Get the sampling frequency."""
        return self._get_iio_dev_attr("sampling_frequency")

    @sample_rate.setter
    def sample_rate(self, rate):
        """Set the sampling frequency."""
        self._set_iio_dev_attr("sampling_frequency", str(rate))

    @property
    def sample_averaging_avail(self):
        """Get list of all the sample averaging values available. Only available in 30bit averaged mode."""
        return self._get_iio_dev_attr("sample_averaging_available")

    @property
    def sample_averaging(self):
        """Get the sample averaging. Only available in 30bit averaged mode."""
        return self._get_iio_dev_attr_str("sample_averaging")

    @sample_averaging.setter
    def sample_averaging(self, n_sample):
        """Set the sample averaging. Only available in 30bit averaged mode."""
        if str(self.sample_averaging) != "OFF":
            if str(n_sample) in str(self.sample_averaging_avail):
                self._set_iio_dev_attr("sample_averaging", str(n_sample))
            else:
                raise ValueError(
                    "Error: Number of avg samples not supported \nUse one of: "
                    + str(self.sample_averaging_avail)
                )
        else:
            raise Exception("Sample Averaging only available in 30bit averaged mode.")

    class _channel(attribute):
        """AD4x30 differential channel."""

        def __init__(self, ctrl, channel_name):
            self.name = channel_name
            self._ctrl = ctrl

        @property
        def calibbias(self):
            """Get calibration bias/offset value."""
            return self._get_iio_attr(self.name, "calibbias", False, self._ctrl)

        @calibbias.setter
        def calibbias(self, calibbias):
            """Set calibration bias/offset value."""
            self._set_iio_attr(
                self.name, "calibbias", False, int(calibbias), self._ctrl
            )

        @property
        def calibscale(self):
            """Get calibration scale value."""
            return self._get_iio_attr(self.name, "calibscale", False, self._ctrl)

        @calibscale.setter
        def calibscale(self, calibscale):
            """Set calibration scale value."""
            self._set_iio_attr(self.name, "calibscale", False, calibscale, self._ctrl)


class adaq42xx(ad4630):

    """ ADAQ4224 is a 24-bit precision SAR ADC data acquisition module """

    _compatible_parts = ["adaq4224", "adaq4216", "adaq4220"]

    def __init__(self, uri="", device_name="adaq4224"):
        super().__init__(uri, device_name)

    class _channel(ad4630._channel):
        """ADAQ42xx differential channel."""

        @property
        def scale_available(self):
            """Provides all available scale(gain) settings for the ADAQ42xx channel"""
            return self._get_iio_attr(self.name, "scale_available", False)

        @property
        def scale(self):
            """ADAQ42xx channel scale"""
            return float(self._get_iio_attr_str(self.name, "scale", False))

        @scale.setter
        def scale(self, value):
            self._set_iio_attr(self.name, "scale", False, str(Decimal(value).real))



# Copyright (C) 2022-2024 Analog Devices, Inc.
#
# SPDX short identifier: ADIBSD

import numpy as np
from adi.attribute import attribute
from adi.context_manager import context_manager
from adi.rx_tx import rx


class max31855(rx, context_manager, attribute):
    """MAX31855 thermocouple device"""

    _device_name = "max31855"
    _rx_data_type = np.int16
    _rx_unbuffered_data = True
    _rx_data_si_type = float

    def __init__(self, uri=""):
        context_manager.__init__(self, uri, self._device_name)
        self._ctrl = self._ctx.find_device("maxim_thermocouple")

        if self._ctrl is None:
            raise Exception("No device found")

        self.temp_i = self._channel(self._ctrl, "i_temp")
        self.temp_t = self._channel(self._ctrl, "t_temp")
        self._rx_channel_names = ["t_temp", "i_temp"]
        rx.__init__(self)

    class _channel(attribute):
        """MAX31855 channel"""

        def __init__(self, ctrl, channel_name):
            self.name = channel_name
            self._ctrl = ctrl

        @property
        def raw(self):
            """MAX31855 channel raw value"""
            return self._get_iio_attr(self.name, "raw", False)

        @property
        def scale(self):
            """MAX31855 channel scale value"""
            return self._get_iio_attr(self.name, "scale", False)



# Copyright (C) 2021-2024 Analog Devices, Inc.
#
# SPDX short identifier: ADIBSD

from adi.attribute import attribute
from adi.context_manager import context_manager
from adi.rx_tx import rx


class adpd188(rx, context_manager):

    """ADPD188 photo-electronic device."""

    channel = []  # type: ignore
    _device_name = ""

    def __init__(self, uri="", device_index=0):
        """ADPD188 class constructor."""
        context_manager.__init__(self, uri, self._device_name)

        compatible_parts = ["adpd188"]

        self._ctrl = None
        index = 0

        # Selecting the device_index-th device from the adpd188 family as working device.
        for device in self._ctx.devices:
            if device.name in compatible_parts:
                if index == device_index:
                    self._ctrl = device
                    self._rxadc = device
                    break
                else:
                    index += 1

        # dynamically get channels and sorting them after the color
        # self._ctrl.channels.sort(key=lambda x: str(x.id[14:]))
        self._rx_channel_names = []
        self.channel = []
        for ch in self._ctrl._channels:
            name = ch._id
            self._rx_channel_names.append(name)
            self.channel.append(self._channel(self._ctrl, name))
        rx.__init__(self)
        self.rx_buffer_size = 16

    @property
    def mode(self):
        """Read mode of operation to device."""
        return self._get_iio_dev_attr_str("mode", self._rxadc)

    @mode.setter
    def mode(self, value):
        self._set_iio_dev_attr_str("mode", value, self._rxadc)

    @property
    def sample_rate(self):
        """Sets sampling frequency of the ADPD188."""
        return self._get_iio_attr(self.channel[0].name, "sampling_frequency", False)

    @sample_rate.setter
    def sample_rate(self, value):
        self._set_iio_attr(self.channel[0].name, "sampling_frequency", False, value)

    class _channel(attribute):

        """ADPD188 channel."""

        def __init__(self, ctrl, channel_name):
            self.name = channel_name
            self._ctrl = ctrl

        @property
        def raw(self):
            """ADPD188 channel raw value."""
            return self._get_iio_attr(self.name, "raw", False)

        @property
        def offset(self):
            """ADPD188 channel offset."""
            return self._get_iio_attr(self.name, "offset", False)

        @offset.setter
        def offset(self, value):
            self._get_iio_attr(self.name, "offset", False, value)

        @property
        def mode_available(self):
            """Read mode of operation to device."""
            return self._get_iio_attr_str(
                self.name, "mode_available", False, self._rxadc
            )



# Copyright (C) 2022-2024 Analog Devices, Inc.
#
# SPDX short identifier: ADIBSD

from adi.attribute import attribute
from adi.context_manager import context_manager


class adf4355(attribute, context_manager):
    """ADF4355 Microwave Wideband Synthesizer with Integrated VCO

    parameters:
        uri: type=string
            URI of IIO context with ADF4355
    """

    _device_name = "adf4355"

    def __init__(self, uri=""):
        context_manager.__init__(self, uri, self._device_name)

        # Find the device
        self._ctrl = self._ctx.find_device(self._device_name)

        # Raise an exception if the device isn't found
        if not self._ctrl:
            raise Exception("ADF4355 device not found")

    @property
    def frequency_altvolt0(self):
        return self._get_iio_attr("altvoltage0", "frequency", True, self._ctrl)

    @frequency_altvolt0.setter
    def frequency_altvolt0(self, value):
        self._set_iio_attr("altvoltage0", "frequency", True, value, self._ctrl)

    @property
    def powerdown_altvolt0(self):
        return bool(
            1 - int(self._get_iio_attr("altvoltage0", "powerdown", True, self._ctrl))
        )

    @powerdown_altvolt0.setter
    def powerdown_altvolt0(self, value):
        self._set_iio_attr("altvoltage0", "powerdown", True, 1 - int(value), self._ctrl)

    @property
    def frequency_altvolt1(self):
        return self._get_iio_attr("altvoltage1", "frequency", True, self._ctrl)

    @frequency_altvolt1.setter
    def frequency_altvolt1(self, value):
        self._set_iio_attr("altvoltage1", "frequency", True, value, self._ctrl)

    @property
    def powerdown_altvolt1(self):
        return bool(
            1 - int(self._get_iio_attr("altvoltage1", "powerdown", True, self._ctrl))
        )

    @powerdown_altvolt1.setter
    def powerdown_altvolt1(self, value):
        self._set_iio_attr("altvoltage1", "powerdown", True, 1 - int(value), self._ctrl)



# Copyright (C) 2019-2024 Analog Devices, Inc.
#
# SPDX short identifier: ADIBSD

from adi.attribute import attribute
from adi.context_manager import context_manager
from adi.rx_tx import rx


class adis16475(rx, context_manager):
    """ADIS16475 Compact, Precision, Six Degrees of Freedom Inertial Sensor"""

    _complex_data = False
    _rx_channel_names = [
        "anglvel_x",
        "anglvel_y",
        "anglvel_z",
        "accel_x",
        "accel_y",
        "accel_z",
        "temp0",
        "deltaangl_x",
        "deltaangl_y",
        "deltaangl_z",
        "deltavelocity_x",
        "deltavelocity_y",
        "deltavelocity_z",
    ]
    _device_name = ""

    def __init__(self, uri="", device_name="adis16505-2"):
        context_manager.__init__(self, uri, self._device_name)

        compatible_parts = [
            "adis16470",
            "adis16475-1",
            "adis16475-2",
            "adis16475-3",
            "adis16477-1",
            "adis16477-2",
            "adis16477-3",
            "adis16465-1",
            "adis16465-2",
            "adis16465-3",
            "adis16467-1",
            "adis16467-2",
            "adis16467-3",
            "adis16500",
            "adis16501",
            "adis16505-1",
            "adis16505-2",
            "adis16505-3",
            "adis16507-1",
            "adis16507-2",
            "adis16507-3",
            "adis16575-2",
            "adis16575-3",
            "adis16576-2",
            "adis16576-3",
            "adis16577-2",
            "adis16577-3",
        ]

        if device_name not in compatible_parts:
            raise Exception(
                "Not a compatible device:"
                + str(device_name)
                + ".Please select from:"
                + str(compatible_parts)
            )
        else:
            self._ctrl = self._ctx.find_device(device_name)
            self._rxadc = self._ctx.find_device(device_name)
            trigger_name = device_name + "-dev0"

        if self._ctrl is None:
            print(
                "No device found with device_name = "
                + device_name
                + ". Searching for a device found in the compatible list."
            )
            for i in compatible_parts:
                self._ctrl = self._ctx.find_device(i)
                self._rxadc = self._ctx.find_device(i)
                if self._ctrl is not None:
                    print("Fond device = " + i + ". Will use this device instead.")
                    trigger_name = i + "-dev0"
                    break
            if self._ctrl is None:
                raise Exception("No compatible device found")

        # Set default trigger
        self._trigger = self._ctx.find_device(trigger_name)
        self._rxadc._set_trigger(self._trigger)

        self.anglvel_x = self._channel_with_offset(self._ctrl, "anglvel_x")
        self.anglvel_y = self._channel_with_offset(self._ctrl, "anglvel_y")
        self.anglvel_z = self._channel_with_offset(self._ctrl, "anglvel_z")
        self.accel_x = self._channel_with_offset(self._ctrl, "accel_x")
        self.accel_y = self._channel_with_offset(self._ctrl, "accel_y")
        self.accel_z = self._channel_with_offset(self._ctrl, "accel_z")
        self.temp = self._channel(self._ctrl, "temp0")
        self.deltaangl_x = self._channel(self._ctrl, "deltaangl_x")
        self.deltaangl_y = self._channel(self._ctrl, "deltaangl_y")
        self.deltaangl_z = self._channel(self._ctrl, "deltaangl_z")
        self.deltavelocity_x = self._channel(self._ctrl, "deltavelocity_x")
        self.deltavelocity_y = self._channel(self._ctrl, "deltavelocity_y")
        self.deltavelocity_z = self._channel(self._ctrl, "deltavelocity_z")

        rx.__init__(self)
        self.rx_buffer_size = 16  # Make default buffer smaller

    def __get_scaled_sensor(self, channel_name: str) -> float:
        raw = self._get_iio_attr(channel_name, "raw", False)
        scale = self._get_iio_attr(channel_name, "scale", False)

        return raw * scale

    def get_anglvel_x(self):
        """Value returned in radians per second."""
        return self.__get_scaled_sensor("anglvel_x")

    anglvel_x_conv = property(get_anglvel_x, None)

    def get_anglvel_y(self):
        """Value returned in radians per second."""
        return self.__get_scaled_sensor("anglvel_y")

    anglvel_y_conv = property(get_anglvel_y, None)

    def get_anglvel_z(self):
        """Value returned in radians per second."""
        return self.__get_scaled_sensor("anglvel_z")

    anglvel_z_conv = property(get_anglvel_z, None)

    def get_accel_x(self):
        """Value returned in meters per squared second."""
        return self.__get_scaled_sensor("accel_x")

    accel_x_conv = property(get_accel_x, None)

    def get_accel_y(self):
        """Value returned in meters per squared second."""
        return self.__get_scaled_sensor("accel_y")

    accel_y_conv = property(get_accel_y, None)

    def get_accel_z(self):
        """Value returned in meters per squared second."""
        return self.__get_scaled_sensor("accel_z")

    accel_z_conv = property(get_accel_z, None)

    def get_deltaangl_x(self):
        """Value returned in radians."""
        return self.__get_scaled_sensor("deltaangl_x")

    deltaangl_x_conv = property(get_deltaangl_x, None)

    def get_deltaangl_y(self):
        """Value returned in radians."""
        return self.__get_scaled_sensor("deltaangl_y")

    deltaangl_y_conv = property(get_deltaangl_y, None)

    def get_deltaangl_z(self):
        """Value returned in radians."""
        return self.__get_scaled_sensor("deltaangl_z")

    deltaangl_z_conv = property(get_deltaangl_z, None)

    def get_deltavelocity_x(self):
        """Value returned in meters per second."""
        return self.__get_scaled_sensor("deltavelocity_x")

    deltavelocity_x_conv = property(get_deltavelocity_x, None)

    def get_deltavelocity_y(self):
        """Value returned in meters per second."""
        return self.__get_scaled_sensor("deltavelocity_y")

    deltavelocity_y_conv = property(get_deltavelocity_y, None)

    def get_deltavelocity_z(self):
        """Value returned in meters per second."""
        return self.__get_scaled_sensor("deltavelocity_z")

    deltavelocity_z_conv = property(get_deltavelocity_z, None)

    def get_temp(self):
        """Value returned in millidegrees Celsius."""
        return self.__get_scaled_sensor("temp0")

    temp_conv = property(get_temp, None)

    @property
    def sample_rate(self):
        """sample_rate: Sample rate in samples per second"""
        return self._get_iio_dev_attr("sampling_frequency")

    @sample_rate.setter
    def sample_rate(self, value):
        self._set_iio_dev_attr_str("sampling_frequency", value)

    @property
    def filter_low_pass_3db_frequency(self):
        """filter_low_pass_3db_frequency: Bandwidth for the accelerometer and
        gyroscope channels"""
        return self._get_iio_dev_attr("filter_low_pass_3db_frequency")

    @filter_low_pass_3db_frequency.setter
    def filter_low_pass_3db_frequency(self, value):
        self._set_iio_dev_attr_str("filter_low_pass_3db_frequency", value)

    @property
    def anglvel_x_calibbias(self):
        """User calibration offset for gyroscope for the x-axis."""
        return self._get_iio_attr("anglvel_x", "calibbias", False)

    @anglvel_x_calibbias.setter
    def anglvel_x_calibbias(self, value):
        self._set_iio_attr("anglvel_x", "calibbias", False, value)

    @property
    def anglvel_y_calibbias(self):
        """User calibration offset for gyroscope for the y-axis."""
        return self._get_iio_attr("anglvel_y", "calibbias", False)

    @anglvel_y_calibbias.setter
    def anglvel_y_calibbias(self, value):
        self._set_iio_attr("anglvel_y", "calibbias", False, value)

    @property
    def anglvel_z_calibbias(self):
        """User calibration offset for gyroscope for the z-axis."""
        return self._get_iio_attr("anglvel_z", "calibbias", False)

    @anglvel_z_calibbias.setter
    def anglvel_z_calibbias(self, value):
        self._set_iio_attr("anglvel_z", "calibbias", False, value)

    @property
    def accel_x_calibbias(self):
        """User calibration offset for accelerometer for the x-axis."""
        return self._get_iio_attr("accel_x", "calibbias", False)

    @accel_x_calibbias.setter
    def accel_x_calibbias(self, value):
        self._set_iio_attr("accel_x", "calibbias", False, value)

    @property
    def accel_y_calibbias(self):
        """User calibration offset for accelerometer for the y-axis."""
        return self._get_iio_attr("accel_y", "calibbias", False)

    @accel_y_calibbias.setter
    def accel_y_calibbias(self, value):
        self._set_iio_attr("accel_y", "calibbias", False, value)

    @property
    def accel_z_calibbias(self):
        """User calibration offset for accelerometer for the z-axis."""
        return self._get_iio_attr("accel_z", "calibbias", False)

    @accel_z_calibbias.setter
    def accel_z_calibbias(self, value):
        self._set_iio_attr("accel_z", "calibbias", False, value)

    @property
    def firmware_revision(self):
        """firmware_revision: the firmware revision for the internal firmware"""
        return self._get_iio_debug_attr_str("firmware_revision")

    @property
    def firmware_date(self):
        """firmware_date: the factory configuration date"""
        return self._get_iio_debug_attr_str("firmware_date")

    @property
    def product_id(self):
        """product_id: the numerical portion of the device number"""
        return self._get_iio_debug_attr("product_id")

    @property
    def serial_number(self):
        """serial_number: lot specific serial number"""
        return self._get_iio_debug_attr_str("serial_number")

    @property
    def flash_count(self):
        """flash_counter: flash memory write count"""
        return self._get_iio_debug_attr("flash_count")

    class _channel_with_offset(attribute):
        """Channel with offset"""

        def __init__(self, ctrl, channel_name):
            self.name = channel_name
            self._ctrl = ctrl

        @property
        def calibbias(self):
            """ADIS165x channel offset"""
            return self._get_iio_attr(self.name, "calibbias", False)

        @calibbias.setter
        def calibbias(self, value):
            self._set_iio_attr(self.name, "calibbias", False, value)

        @property
        def raw(self):
            """ADIS165x channel raw value"""
            return self._get_iio_attr(self.name, "raw", False)

        @property
        def scale(self):
            """ADIS165x channel scale(gain)"""
            return float(self._get_iio_attr_str(self.name, "scale", False))

    class _channel(attribute):
        """Channel"""

        def __init__(self, ctrl, channel_name):
            self.name = channel_name
            self._ctrl = ctrl

        @property
        def raw(self):
            """ADIS165x channel raw value"""
            return self._get_iio_attr(self.name, "raw", False)

        @property
        def scale(self):
            """ADIS165x channel scale(gain)"""
            return float(self._get_iio_attr_str(self.name, "scale", False))

    def reg_read(self, reg):
        """Direct Register Access via debugfs"""
        self._set_iio_debug_attr_str("direct_reg_access", reg, self._ctrl)
        return self._get_iio_debug_attr_str("direct_reg_access", self._ctrl)

    def reg_write(self, reg, value):
        """Direct Register Access via debugfs"""
        self._set_iio_debug_attr_str("direct_reg_access", f"{reg} {value}", self._ctrl)



# Copyright (C) 2019-2024 Analog Devices, Inc.
#
# SPDX short identifier: ADIBSD

from adi.context_manager import context_manager
from adi.rx_tx import rx
from adi.sync_start import sync_start


class ad9680(rx, context_manager, sync_start):
    """ AD9680 High-Speed ADC """

    _complex_data = False
    _rx_channel_names = ["voltage0", "voltage1"]
    _device_name = ""

    def __init__(self, uri=""):

        context_manager.__init__(self, uri, self._device_name)

        self._rxadc = self._ctx.find_device("axi-ad9680-hpc")

        rx.__init__(self)

    @property
    def test_mode(self):
        """test_mode: Select Test Mode. Options are:
        off midscale_short pos_fullscale neg_fullscale checkerboard pn_long pn_short one_zero_toggle user ramp"""
        return self._get_iio_attr("voltage0", "test_mode", False)

    @test_mode.setter
    def test_mode(self, value):
        self._set_iio_attr("voltage0", "test_mode", False, value, self._rxadc)



# Copyright (C) 2019-2024 Analog Devices, Inc.
#
# SPDX short identifier: ADIBSD

from adi.ad9144 import ad9144
from adi.ad9680 import ad9680


class DAQ2(ad9144, ad9680):
    """ DAQ2 High-Speed Data Aquistion Device """

    def __init__(self, uri=""):

        ad9144.__init__(self, uri=uri)
        ad9680.__init__(self, uri=uri)



# Copyright (C) 2019-2024 Analog Devices, Inc.
#
# SPDX short identifier: ADIBSD

from abc import ABC, abstractmethod

from adi.attribute import attribute
from adi.context_manager import context_manager
from adi.rx_tx import rx


class adis16XXX(rx, context_manager, ABC):

    _complex_data = False

    _rx_channel_names = [
        "anglvel_x",
        "anglvel_y",
        "anglvel_z",
        "accel_x",
        "accel_y",
        "accel_z",
        "temp0",
    ]

    _device_name = ""

    """Disable mapping of trigger to RX device."""
    disable_trigger = False

    # @property
    @abstractmethod
    def compatible_parts(self):
        raise NotImplementedError

    def __init__(self, uri="", device_name=None, trigger_name=None):
        context_manager.__init__(self, uri, self._device_name)

        if not device_name:
            device_name = self.compatible_parts[0]

        if device_name not in self.compatible_parts:
            raise Exception(
                "Not a compatible device:"
                + str(device_name)
                + ".Please select from:"
                + str(self.compatible_parts)
            )
        else:
            self._ctrl = self._ctx.find_device(device_name)
            self._rxadc = self._ctx.find_device(device_name)
            if not trigger_name:
                trigger_name = device_name + "-dev0"

        if self._ctrl is None:
            print(
                "No device found with device_name = "
                + device_name
                + ". Searching for a device found in the compatible list."
            )
            for i in self.compatible_parts:
                self._ctrl = self._ctx.find_device(i)
                self._rxadc = self._ctx.find_device(i)
                if self._ctrl is not None:
                    print("Fond device = " + i + ". Will use this device instead.")
                    break
            if self._ctrl is None:
                raise Exception("No compatible device found")

        self.anglvel_x = self._anglvel_accel_channels(self._ctrl, "anglvel_x")
        self.anglvel_y = self._anglvel_accel_channels(self._ctrl, "anglvel_y")
        self.anglvel_z = self._anglvel_accel_channels(self._ctrl, "anglvel_z")
        self.accel_x = self._anglvel_accel_channels(self._ctrl, "accel_x")
        self.accel_y = self._anglvel_accel_channels(self._ctrl, "accel_y")
        self.accel_z = self._anglvel_accel_channels(self._ctrl, "accel_z")
        self.temp = self._temp_channel(self._ctrl, "temp0")
        self.deltaangl_x = self._delta_channels(self._ctrl, "deltaangl_x")
        self.deltaangl_y = self._delta_channels(self._ctrl, "deltaangl_y")
        self.deltaangl_z = self._delta_channels(self._ctrl, "deltaangl_z")
        self.deltavelocity_x = self._delta_channels(self._ctrl, "deltavelocity_x")
        self.deltavelocity_y = self._delta_channels(self._ctrl, "deltavelocity_y")
        self.deltavelocity_z = self._delta_channels(self._ctrl, "deltavelocity_z")

        # Set default trigger
        if not self.disable_trigger:
            self._trigger = self._ctx.find_device(trigger_name)
            self._rxadc._set_trigger(self._trigger)

        rx.__init__(self)
        self.rx_buffer_size = 16  # Make default buffer smaller

    def __get_scaled_sensor(self, channel_name: str) -> float:
        raw = self._get_iio_attr(channel_name, "raw", False)
        scale = self._get_iio_attr(channel_name, "scale", False)

        return raw * scale

    def __get_scaled_sensor_temp(self, channel_name: str) -> float:
        raw = self._get_iio_attr(channel_name, "raw", False)
        scale = self._get_iio_attr(channel_name, "scale", False)
        offset = self._get_iio_attr(channel_name, "offset", False)

        return (raw + offset) * scale

    def get_anglvel_x(self):
        """Value returned in radians per second."""
        return self.__get_scaled_sensor("anglvel_x")

    anglvel_x_conv = property(get_anglvel_x, None)

    def get_anglvel_y(self):
        """Value returned in radians per second."""
        return self.__get_scaled_sensor("anglvel_y")

    anglvel_y_conv = property(get_anglvel_y, None)

    def get_anglvel_z(self):
        """Value returned in radians per second."""
        return self.__get_scaled_sensor("anglvel_z")

    anglvel_z_conv = property(get_anglvel_z, None)

    def get_accel_x(self):
        """Value returned in meters per squared second."""
        return self.__get_scaled_sensor("accel_x")

    accel_x_conv = property(get_accel_x, None)

    def get_accel_y(self):
        """Value returned in meters per squared second."""
        return self.__get_scaled_sensor("accel_y")

    accel_y_conv = property(get_accel_y, None)

    def get_accel_z(self):
        """Value returned in meters per squared second."""
        return self.__get_scaled_sensor("accel_z")

    accel_z_conv = property(get_accel_z, None)

    def get_temp(self):
        """Value returned in millidegrees Celsius."""
        return self.__get_scaled_sensor_temp("temp0")

    temp_conv = property(get_temp, None)

    def get_deltaangl_x(self):
        """Value returned in radians."""
        return self.__get_scaled_sensor("deltaangl_x")

    deltaangl_x_conv = property(get_deltaangl_x, None)

    def get_deltaangl_y(self):
        """Value returned in radians."""
        return self.__get_scaled_sensor("deltaangl_y")

    deltaangl_y_conv = property(get_deltaangl_y, None)

    def get_deltaangl_z(self):
        """Value returned in radians."""
        return self.__get_scaled_sensor("deltaangl_z")

    deltaangl_z_conv = property(get_deltaangl_z, None)

    def get_deltavelocity_x(self):
        """Value returned in meters per second."""
        return self.__get_scaled_sensor("deltavelocity_x")

    deltavelocity_x_conv = property(get_deltavelocity_x, None)

    def get_deltavelocity_y(self):
        """Value returned in meters per second."""
        return self.__get_scaled_sensor("deltavelocity_y")

    deltavelocity_y_conv = property(get_deltavelocity_y, None)

    def get_deltavelocity_z(self):
        """Value returned in meters per second."""
        return self.__get_scaled_sensor("deltavelocity_z")

    deltavelocity_z_conv = property(get_deltavelocity_z, None)

    @property
    def sample_rate(self):
        """sample_rate: Sample rate in samples per second"""
        return self._get_iio_dev_attr("sampling_frequency")

    @sample_rate.setter
    def sample_rate(self, value):
        self._set_iio_dev_attr_str("sampling_frequency", value)

    @property
    def current_timestamp_clock(self):
        """current_timestamp_clock: Source clock for timestamps"""
        return self._get_iio_dev_attr("current_timestamp_clock")

    @current_timestamp_clock.setter
    def current_timestamp_clock(self, value):
        self._set_iio_dev_attr_str("current_timestamp_clock", value)

    @property
    def anglvel_x_calibbias(self):
        """User calibration offset for gyroscope for the x-axis."""
        return self._get_iio_attr("anglvel_x", "calibbias", False)

    @anglvel_x_calibbias.setter
    def anglvel_x_calibbias(self, value):
        self._set_iio_attr("anglvel_x", "calibbias", False, value)

    @property
    def anglvel_y_calibbias(self):
        """User calibration offset for gyroscope for the y-axis."""
        return self._get_iio_attr("anglvel_y", "calibbias", False)

    @anglvel_y_calibbias.setter
    def anglvel_y_calibbias(self, value):
        self._set_iio_attr("anglvel_y", "calibbias", False, value)

    @property
    def anglvel_z_calibbias(self):
        """User calibration offset for gyroscope for the z-axis."""
        return self._get_iio_attr("anglvel_z", "calibbias", False)

    @anglvel_z_calibbias.setter
    def anglvel_z_calibbias(self, value):
        self._set_iio_attr("anglvel_z", "calibbias", False, value)

    @property
    def accel_x_calibbias(self):
        """User calibration offset for accelerometer for the x-axis."""
        return self._get_iio_attr("accel_x", "calibbias", False)

    @accel_x_calibbias.setter
    def accel_x_calibbias(self, value):
        self._set_iio_attr("accel_x", "calibbias", False, value)

    @property
    def accel_y_calibbias(self):
        """User calibration offset for accelerometer for the y-axis."""
        return self._get_iio_attr("accel_y", "calibbias", False)

    @accel_y_calibbias.setter
    def accel_y_calibbias(self, value):
        self._set_iio_attr("accel_y", "calibbias", False, value)

    @property
    def accel_z_calibbias(self):
        """User calibration offset for accelerometer for the z-axis."""
        return self._get_iio_attr("accel_z", "calibbias", False)

    @accel_z_calibbias.setter
    def accel_z_calibbias(self, value):
        self._set_iio_attr("accel_z", "calibbias", False, value)

    #####
    @property
    def anglvel_x_calibscale(self):
        """Calibscale value for gyroscope for the x-axis."""
        return self._get_iio_attr("anglvel_x", "calibscale", False)

    @anglvel_x_calibscale.setter
    def anglvel_x_calibscale(self, value):
        self._set_iio_attr("anglvel_x", "calibscale", False, value)

    @property
    def anglvel_y_calibscale(self):
        """Calibscale value for gyroscope for the y-axis."""
        return self._get_iio_attr("anglvel_y", "calibscale", False)

    @anglvel_y_calibscale.setter
    def anglvel_y_calibscale(self, value):
        self._set_iio_attr("anglvel_y", "calibscale", False, value)

    @property
    def anglvel_z_calibscale(self):
        """Calibscale value for gyroscope for the z-axis."""
        return self._get_iio_attr("anglvel_z", "calibscale", False)

    @anglvel_z_calibscale.setter
    def anglvel_z_calibscale(self, value):
        self._set_iio_attr("anglvel_z", "calibscale", False, value)

    @property
    def accel_x_calibscale(self):
        """Calibscale value for accelerometer for the x-axis."""
        return self._get_iio_attr("accel_x", "calibscale", False)

    @accel_x_calibscale.setter
    def accel_x_calibscale(self, value):
        self._set_iio_attr("accel_x", "calibscale", False, value)

    @property
    def accel_y_calibscale(self):
        """Calibcale value for accelerometer for the y-axis."""
        return self._get_iio_attr("accel_y", "calibscale", False)

    @accel_y_calibscale.setter
    def accel_y_calibscale(self, value):
        self._set_iio_attr("accel_y", "calibscale", False, value)

    @property
    def accel_z_calibscale(self):
        """Calibscale for accelerometer for the z-axis."""
        return self._get_iio_attr("accel_z", "calibscale", False)

    @accel_z_calibscale.setter
    def accel_z_calibscale(self, value):
        self._set_iio_attr("accel_z", "calibscale", False, value)

    @property
    def anglvel_x_filter_low_pass_3db_frequency(self):
        """Bandwidth for gyroscope for the x-axis."""
        return self._get_iio_attr("anglvel_x", "filter_low_pass_3db_frequency", False)

    @anglvel_x_filter_low_pass_3db_frequency.setter
    def anglvel_x_filter_low_pass_3db_frequency(self, value):
        self._set_iio_attr("anglvel_x", "filter_low_pass_3db_frequency", False, value)

    @property
    def anglvel_y_filter_low_pass_3db_frequency(self):
        """Bandwidth for gyroscope for the y-axis."""
        return self._get_iio_attr("anglvel_y", "filter_low_pass_3db_frequency", False)

    @anglvel_y_filter_low_pass_3db_frequency.setter
    def anglvel_y_filter_low_pass_3db_frequency(self, value):
        self._set_iio_attr("anglvel_y", "filter_low_pass_3db_frequency", False, value)

    @property
    def anglvel_z_filter_low_pass_3db_frequency(self):
        """Bandwidth for gyroscope for the z-axis."""
        return self._get_iio_attr("anglvel_z", "filter_low_pass_3db_frequency", False)

    @anglvel_z_filter_low_pass_3db_frequency.setter
    def anglvel_z_filter_low_pass_3db_frequency(self, value):
        self._set_iio_attr("anglvel_z", "filter_low_pass_3db_frequency", False, value)

    @property
    def accel_x_filter_low_pass_3db_frequency(self):
        """Bandwidth for accelerometer for the x-axis."""
        return self._get_iio_attr("accel_x", "filter_low_pass_3db_frequency", False)

    @accel_x_filter_low_pass_3db_frequency.setter
    def accel_x_filter_low_pass_3db_frequency(self, value):
        self._set_iio_attr("accel_x", "filter_low_pass_3db_frequency", False, value)

    @property
    def accel_y_filter_low_pass_3db_frequency(self):
        """Bandwidth for accelerometer for the y-axis."""
        return self._get_iio_attr("accel_y", "filter_low_pass_3db_frequency", False)

    @accel_y_filter_low_pass_3db_frequency.setter
    def accel_y_filter_low_pass_3db_frequency(self, value):
        self._set_iio_attr("accel_y", "filter_low_pass_3db_frequency", False, value)

    @property
    def accel_z_filter_low_pass_3db_frequency(self):
        """Bandwidth for accelerometer for the z-axis."""
        return self._get_iio_attr("accel_z", "filter_low_pass_3db_frequency", False)

    @accel_z_filter_low_pass_3db_frequency.setter
    def accel_z_filter_low_pass_3db_frequency(self, value):
        self._set_iio_attr("accel_z", "filter_low_pass_3db_frequency", False, value)

    @property
    def firmware_revision(self):
        """firmware_revision: the firmware revision for the internal firmware"""
        return self._get_iio_debug_attr_str("firmware_revision")

    @property
    def firmware_date(self):
        """firmware_date: the factory configuration date"""
        return self._get_iio_debug_attr_str("firmware_date")

    @property
    def product_id(self):
        """product_id: the numerical portion of the device number"""
        return self._get_iio_debug_attr("product_id")

    @property
    def serial_number(self):
        """serial_number: lot specific serial number"""
        return self._get_iio_debug_attr_str("serial_number")

    @property
    def flash_count(self):
        """flash_counter: flash memory write count"""
        return self._get_iio_debug_attr("flash_count")

    class _temp_channel(attribute):
        """ADIS16480 temperature channel."""

        def __init__(self, ctrl, channel_name):
            self.name = channel_name
            self._ctrl = ctrl

        @property
        def raw(self):
            """ADIS16480 raw value"""
            return self._get_iio_attr(self.name, "raw", False)

        @property
        def scale(self):
            """ADIS16480 scale value"""
            return self._get_iio_attr(self.name, "scale", False)

        @property
        def offset(self):
            """ADIS16480 offset value"""
            return self._get_iio_attr(self.name, "offset", False)

    class _pressure_channel(attribute):
        """ADIS16480 pressure channel."""

        def __init__(self, ctrl, channel_name):
            self.name = channel_name
            self._ctrl = ctrl

        @property
        def raw(self):
            """ADIS16480 raw value"""
            return self._get_iio_attr(self.name, "raw", False)

        @property
        def scale(self):
            """ADIS16480 scale value"""
            return self._get_iio_attr(self.name, "scale", False)

        @property
        def calibbias(self):
            """ADIS16480 calibration offset"""
            return self._get_iio_attr(self.name, "calibbias", False)

        @calibbias.setter
        def calibbias(self, value):
            self._set_iio_attr(self.name, "calibbias", False, value)

    class _magn_channel(_pressure_channel):
        """ADIS16480 magnetometer channel."""

        def __init__(self, ctrl, channel_name):
            self.name = channel_name
            self._ctrl = ctrl

        @property
        def filter_low_pass_3db_frequency(self):
            """ADIS16480 channel bandwidth"""
            return self._get_iio_attr(self.name, "filter_low_pass_3db_frequency", False)

        @filter_low_pass_3db_frequency.setter
        def filter_low_pass_3db_frequency(self, value):
            self._set_iio_attr(self.name, "filter_low_pass_3db_frequency", False, value)

    class _anglvel_accel_channels(_magn_channel):
        """ADIS16480 gyro and accelerometer channels."""

        def __init__(self, ctrl, channel_name):
            self.name = channel_name
            self._ctrl = ctrl

        @property
        def calibscale(self):
            """ADIS16480 calibscale value"""
            return self._get_iio_attr(self.name, "calibscale", False)

        @calibscale.setter
        def calibscale(self, value):
            self._set_iio_attr(self.name, "calibscale", False, value)

    class _delta_channels(attribute):
        """ADIS16480 delta angle and delta velocity channels."""

        def __init__(self, ctrl, channel_name):
            self.name = channel_name
            self._ctrl = ctrl


class adis16XXX_with_delta_angl(adis16XXX):
    def __init__(self, uri="", device_name=None, trigger_name=None):
        adis16XXX.__init__(self, uri, device_name, trigger_name)

        self._rx_channel_names.append("deltaangl_x")
        self._rx_channel_names.append("deltaangl_y")
        self._rx_channel_names.append("deltaangl_z")
        self._rx_channel_names.append("deltavelocity_x")
        self._rx_channel_names.append("deltavelocity_y")
        self._rx_channel_names.append("deltavelocity_z")


class adis16XXX_with_mag(adis16XXX):
    def __init__(self, uri="", device_name=None, trigger_name=None):
        adis16XXX.__init__(self, uri, device_name, trigger_name)

        self._rx_channel_names.append("pressure0")
        self._rx_channel_names.append("magn_x")
        self._rx_channel_names.append("magn_y")
        self._rx_channel_names.append("magn_z")
        self.pressure = self._pressure_channel(self._ctrl, "pressure0")
        self.magn_x = self._magn_channel(self._ctrl, "magn_x")
        self.magn_y = self._magn_channel(self._ctrl, "magn_y")
        self.magn_z = self._magn_channel(self._ctrl, "magn_z")

    def get_magn_x(self):
        """Value returned in radians."""
        return self.__get_scaled_sensor("magn_x")

    magn_x_conv = property(get_magn_x, None)

    def get_magn_y(self):
        """Value returned in radians."""
        return self.__get_scaled_sensor("magn_y")

    magn_y_conv = property(get_magn_y, None)

    def get_magn_z(self):
        """Value returned in radians."""
        return self.__get_scaled_sensor("magn_z")

    magn_z_conv = property(get_magn_z, None)

    def get_pressure(self):
        """Value returned in kilo Pascal."""
        return self.__get_scaled_sensor("pressure0")

    pressure_conv = property(get_pressure, None)

    @property
    def magn_x_calibbias(self):
        """User calibration offset for magnetometer for the x-axis."""
        if self.magn_x:
            return self._get_iio_attr("magn_x", "calibbias", False)
        return 0

    @magn_x_calibbias.setter
    def magn_x_calibbias(self, value):
        if self.magn_x:
            self._set_iio_attr("magn_x", "calibbias", False, value)

    @property
    def magn_y_calibbias(self):
        """User calibration offset for magnetometer for the y-axis."""
        if self.magn_y:
            return self._get_iio_attr("magn_y", "calibbias", False)
        return 0

    @magn_y_calibbias.setter
    def magn_y_calibbias(self, value):
        if self.magn_y:
            self._set_iio_attr("magn_y", "calibbias", False, value)

    @property
    def magn_z_calibbias(self):
        """User calibration offset for magnetometer for the z-axis."""
        if self.magn_z:
            return self._get_iio_attr("magn_z", "calibbias", False)
        return 0

    @magn_z_calibbias.setter
    def magn_z_calibbias(self, value):
        if self.magn_z:
            self._set_iio_attr("magn_z", "calibbias", False, value)

    @property
    def pressure_calibbias(self):
        """User calibration offset for pressure."""
        if self.pressure:
            return self._get_iio_attr("pressure0", "calibbias", False)
        return 0

    @pressure_calibbias.setter
    def pressure_calibbias(self, value):
        if self.pressure:
            self._set_iio_attr("pressure0", "calibbias", False, value)

    @property
    def magn_x_filter_low_pass_3db_frequency(self):
        """Bandwidth for magnetometer for the x-axis."""
        if self.magn_x:
            return self._get_iio_attr("magn_x", "filter_low_pass_3db_frequency", False)
        return 0

    @magn_x_filter_low_pass_3db_frequency.setter
    def magn_x_filter_low_pass_3db_frequency(self, value):
        if self.magn_x:
            self._set_iio_attr("magn_x", "filter_low_pass_3db_frequency", False, value)

    @property
    def magn_y_filter_low_pass_3db_frequency(self):
        """Bandwidth for magnetometer for the y-axis."""
        if self.magn_y:
            return self._get_iio_attr("magn_y", "filter_low_pass_3db_frequency", False)
        return 0

    @magn_y_filter_low_pass_3db_frequency.setter
    def magn_y_filter_low_pass_3db_frequency(self, value):
        if self.magn_y:
            self._set_iio_attr("magn_y", "filter_low_pass_3db_frequency", False, value)

    @property
    def magn_z_filter_low_pass_3db_frequency(self):
        """Bandwidth for magnetometer for the z-axis."""
        if self.magn_z:
            return self._get_iio_attr("magn_z", "filter_low_pass_3db_frequency", False)
        return 0

    @magn_z_filter_low_pass_3db_frequency.setter
    def magn_z_filter_low_pass_3db_frequency(self, value):
        if self.magn_z:
            self._set_iio_attr("magn_z", "filter_low_pass_3db_frequency", False, value)


11


class adis16375(adis16XXX):
    """ADIS16375 Low Profile, Low Noise Six Degrees of Freedom Inertial Sensor

    Args:
        uri: URI of IIO context with ADIS16375 device
        device_name: Name of the device in the IIO context. Default is adis16375
        trigger_name: Name of the trigger in the IIO context. Default is adis16375-dev0
    """

    compatible_parts = ["adis16375"]


class adis16480(adis16XXX_with_mag):
    """ADIS16480 Ten Degrees of Freedom Inertial Sensor with Dynamic Orientation Outputs

    Args:
        uri: URI of IIO context with ADIS16480 device
        device_name: Name of the device in the IIO context. Default is adis16480
        trigger_name: Name of the trigger in the IIO context. Default is adis16480-dev0
    """

    compatible_parts = ["adis16480"]


class adis16485(adis16XXX):
    """ADIS16485 Tactical Grade Six Degrees of Freedom MEMS Inertial Sensor

    Args:
        uri: URI of IIO context with ADIS16485 device
        device_name: Name of the device in the IIO context. Default is adis16485
        trigger_name: Name of the trigger in the IIO context. Default is adis16485-dev0
    """

    compatible_parts = ["adis16485"]


class adis16488(adis16XXX_with_mag):
    """ADIS16488  Tactical Grade Ten Degrees of Freedom Inertial Sensor

    Args:
        uri: URI of IIO context with ADIS16488 device
        device_name: Name of the device in the IIO context. Default is adis16488
        trigger_name: Name of the trigger in the IIO context. Default is adis16488-dev0
    """

    compatible_parts = ["adis16488"]


class adis16490(adis16XXX):
    """ADIS16490 Tactical Grade, Six Degrees of Freedom Inertial Sensor

    Args:
        uri: URI of IIO context with ADIS16490 device
        device_name: Name of the device in the IIO context. Default is adis16490
        trigger_name: Name of the trigger in the IIO context. Default is adis16490-dev0
    """

    compatible_parts = ["adis16490"]


class adis16495(adis16XXX):
    """ADIS16495-X Tactical Grade, Six Degrees of Freedom Inertial Sensor

    This class is compatible with the following parts:
    - adis16495-1
    - adis16495-2
    - adis16495-3

    Args:
        uri: URI of IIO context with ADIS16495 device
        device_name: Name of the device in the IIO context. Default is adis16495-1
        trigger_name: Name of the trigger in the IIO context. Default is adis16495-1-dev0
    """

    compatible_parts = ["adis16495-1", "adis16495-2", "adis16495-3"]


class adis16497(adis16XXX):
    """ADIS16497-X Ten Degrees of Freedom Inertial Sensor with Dynamic Orientation Outputs

    This class is compatible with the following parts:
    - adis16497-1
    - adis16497-2
    - adis16497-3
    """

    compatible_parts = ["adis16497-1", "adis16497-2", "adis16497-3"]


class adis16545(adis16XXX_with_delta_angl):
    """ADIS16545-X Tactical Grade, Six Degrees of Freedom Inertial Sensor

    This class is compatible with the following parts:
    - adis16545-1
    - adis16545-2
    - adis16545-3

    Args:
        uri: URI of IIO context with ADIS16545 device
        device_name: Name of the device in the IIO context. Default is adis16545-1
        trigger_name: Name of the trigger in the IIO context. Default is adis16545-1-dev0
    """

    compatible_parts = ["adis16545-1", "adis16545-2", "adis16545-3"]


class adis16547(adis16XXX_with_delta_angl):
    """ADIS16547-X Tactical Grade, Six Degrees of Freedom Inertial Sensors

    This class is compatible with the following parts:
    - adis16547-1
    - adis16547-2
    - adis16547-3

    Args:
        uri: URI of IIO context with ADIS16547 device
        device_name: Name of the device in the IIO context. Default is adis16547-1
        trigger_name: Name of the trigger in the IIO context. Default is adis16547-1-dev0
    """

    compatible_parts = ["adis16547-1", "adis16547-2", "adis16547-3"]



# Copyright (C) 2021-2024 Analog Devices, Inc.
#
# SPDX short identifier: ADIBSD


from adi.attribute import attribute
from adi.context_manager import context_manager


class adf4159(context_manager, attribute):

    """ ADF4159 is a 13 GHz Fractional-N Frequency Synthesizer """

    _device_name = "adf4159"

    def __init__(self, uri=""):

        context_manager.__init__(self, uri, self._device_name)
        # Find the device
        self._ctrl = self._ctx.find_device(self._device_name)
        # Raise an exception if the device isn't found
        if not self._ctrl:
            raise Exception("ADF4159 device not found")

    @property
    def ramp_mode(self):
        """Get/Set the Ramp output mode."""
        return self._get_iio_attr_str("altvoltage0", "ramp_mode", True, self._ctrl)

    @ramp_mode.setter
    def ramp_mode(self, value):
        """Get/Set the Ramp output mode."""

        valid = self._get_iio_attr_str(
            "altvoltage0", "ramp_mode_available", True, self._ctrl
        )
        if value not in valid:

            raise ValueError(
                f'ramp_mode of "{value}" is invalid. Valid options: "{valid}"'
            )

        self._set_iio_attr("altvoltage0", "ramp_mode", True, value, self._ctrl)

    @property
    def enable(self):
        """Get/Set the enable status of the RF output."""
        return bool(
            int(self._get_iio_attr("altvoltage0", "powerdown", True, self._ctrl))
        )

    @enable.setter
    def enable(self, value):
        """Get/Set the enable status of the RF output."""
        self._set_iio_attr("altvoltage0", "powerdown", True, int(value), self._ctrl)

    @property
    def frequency(self):
        """Get/Set the Output Frequency of PLL."""
        return self._get_iio_attr("altvoltage0", "frequency", True, self._ctrl)

    @frequency.setter
    def frequency(self, value):
        """Get/Set the Output Frequency of PLL."""
        self._set_iio_attr("altvoltage0", "frequency", True, value, self._ctrl)

    @property
    def freq_dev_range(self):
        """Get/Set the PLL frequency deviation range."""
        return self._get_iio_attr(
            "altvoltage0", "frequency_deviation_range", True, self._ctrl
        )

    @freq_dev_range.setter
    def freq_dev_range(self, value):
        """Get/Set the PLL frequency deviation range."""
        self._set_iio_attr(
            "altvoltage0", "frequency_deviation_range", True, value, self._ctrl
        )

    @property
    def freq_dev_step(self):
        """Get/Set the PLL frequency deviation step."""
        return self._get_iio_attr(
            "altvoltage0", "frequency_deviation_step", True, self._ctrl
        )

    @freq_dev_step.setter
    def freq_dev_step(self, value):
        """Get/Set the PLL frequency deviation step."""
        self._set_iio_attr(
            "altvoltage0", "frequency_deviation_step", True, (1 + value), self._ctrl
        )

    @property
    def freq_dev_time(self):
        """Get/Set the PLL frequency deviation time."""
        return self._get_iio_attr(
            "altvoltage0", "frequency_deviation_time", True, self._ctrl
        )

    @freq_dev_time.setter
    def freq_dev_time(self, value):
        """Get/Set the PLL frequency deviation time."""
        self._set_iio_attr(
            "altvoltage0", "frequency_deviation_time", True, value, self._ctrl
        )

    @property
    def phase_value(self):
        """Get/Set the PLL frequency deviation time."""
        return self._get_iio_debug_attr("adi,phase", self._ctrl)

    @phase_value.setter
    def phase_value(self, value):
        """Get/Set the PLL frequency deviation time."""
        self._set_iio_debug_attr_str("adi,phase", str(value), self._ctrl)

    @property
    def trig_delay_en(self):
        """Get/Set the txdata-trigger-delay-enable."""
        return self._get_iio_debug_attr("adi,txdata-trigger-delay-enable", self._ctrl)

    @trig_delay_en.setter
    def trig_delay_en(self, value):
        """Get/Set the txdata-trigger-delay-enable."""
        self._set_iio_debug_attr_str(
            "adi,txdata-trigger-delay-enable", str(value), self._ctrl
        )

    @property
    def sing_ful_tri(self):
        """Get/Set Single-full-triangle-enable."""
        return self._get_iio_debug_attr("adi,single-full-triangle-enable", self._ctrl)

    @sing_ful_tri.setter
    def sing_ful_tri(self, value):
        """Get/Set Single-full-triangle-enable."""
        self._set_iio_debug_attr_str(
            "adi,single-full-triangle-enable", str(value), self._ctrl
        )

    @property
    def tx_trig_en(self):
        """Get/Set tx data trigger enable."""
        return self._get_iio_debug_attr("adi,txdata-trigger-enable", self._ctrl)

    @tx_trig_en.setter
    def tx_trig_en(self, value):
        """Get/Set tx data trigger enable."""
        self._set_iio_debug_attr_str(
            "adi,txdata-trigger-enable", str(value), self._ctrl
        )

    @property
    def ramp_delay_en(self):
        """Get/Set ramp delay enable."""
        return self._get_iio_debug_attr("adi,ramp-delay-enable", self._ctrl)

    @ramp_delay_en.setter
    def ramp_delay_en(self, value):
        """Get/Set ramp delay enable."""
        self._set_iio_debug_attr_str("adi,ramp-delay-enable", str(value), self._ctrl)

    @property
    def delay_clk(self):
        """Get/Set the clk delay enable."""
        return self._get_iio_debug_attr(
            "adi,delay-clk-sel-pfd-x-clk1-enable", self._ctrl
        )

    @delay_clk.setter
    def delay_clk(self, value):
        """Get/Set the clk delay enable."""
        self._set_iio_debug_attr_str(
            "adi,delay-clk-sel-pfd-x-clk1-enable", str(value), self._ctrl
        )

    @property
    def delay_start_en(self):
        """Get/Set the delay start enable."""
        return self._get_iio_debug_attr("adi,delay-start-enable", self._ctrl)

    @delay_start_en.setter
    def delay_start_en(self, value):
        """Get/Set the delay start enable."""
        self._set_iio_debug_attr_str("adi,delay-start-enable", str(value), self._ctrl)

    @property
    def delay_word(self):
        """Get/Set the delay word."""
        return self._get_iio_debug_attr("adi,delay-start-word", self._ctrl)

    @delay_word.setter
    def delay_word(self, value):
        """Get/Set the delay word."""
        self._set_iio_debug_attr_str("adi,delay-start-word", str(value), self._ctrl)

    @property
    def clk1_mode(self):
        """Get/Set the mode of 1st clk."""
        return self._get_iio_debug_attr("adi,clk-div-mode", self._ctrl)

    @clk1_mode.setter
    def clk1_mode(self, value):
        """Get/Set the mode of 1st clk."""
        self._set_iio_debug_attr_str("adi,clk-div-mode", str(value), self._ctrl)

    @property
    def ramp_en(self):
        """Get/Set the ramp enable."""
        return self._get_iio_debug_attr("adi,ramp-enable", self._ctrl)

    @ramp_en.setter
    def ramp_en(self, value):
        """Get/Set the ramp enable."""
        self._set_iio_debug_attr_str("adi,ramp-enable", str(value), self._ctrl)

    @property
    def clk1_div_value(self):
        """Get/Set the PLL frequency deviation time."""
        return self._get_iio_debug_attr("adi,clk1-div", self._ctrl)

    @clk1_div_value.setter
    def clk1_div_value(self, value):
        """Get/Set the PLL frequency deviation time."""
        self._set_iio_debug_attr_str("adi,clk1-div", str(value), self._ctrl)

    @property
    def clk2_div_value(self):
        """Get/Set the PLL frequency deviation time."""
        return self._get_iio_debug_attr("adi,clk2-timer-div", self._ctrl)

    @clk2_div_value.setter
    def clk2_div_value(self, value):
        """Get/Set the PLL frequency deviation time."""
        self._set_iio_debug_attr_str("adi,clk2-timer-div", str(value), self._ctrl)

    @property
    def muxout_sel(self):
        """Get/Set the PLL frequency deviation time."""
        return self._get_iio_debug_attr("adi,muxout-select", self._ctrl)

    @muxout_sel.setter
    def muxout_sel(self, value):
        """Get/Set the PLL frequency deviation time."""
        self._set_iio_debug_attr_str("adi,muxout-select", str(value), self._ctrl)



# Copyright (C) 2019-2024 Analog Devices, Inc.
#
# SPDX short identifier: ADIBSD

from typing import List

from adi.ad9081 import _sortconv
from adi.context_manager import context_manager
from adi.rx_tx import tx
from adi.sync_start import sync_start


class ad9172(tx, context_manager, sync_start):
    """AD9172 High-Speed DAC"""

    _complex_data = True
    _tx_channel_names: List[str] = []
    _device_name = ""

    def __init__(self, uri=""):

        context_manager.__init__(self, uri, self._device_name)

        self._txdac = self._ctx.find_device("axi-ad9172-hpc")
        if not self._txdac:
            raise RuntimeError("Could not find axi-ad9172-hpc")

        self._tx_channel_names = []
        for chan in self._txdac.channels:
            if (
                hasattr(chan, "scan_element")
                and chan.scan_element
                and chan.id not in self._tx_channel_names
            ):
                self._tx_channel_names.append(chan.id)

        self._tx_channel_names = _sortconv(self._tx_channel_names, complex=True)

        tx.__init__(self)

    @property
    def sample_rate(self):
        """sample_rate: Sample rate TX path in samples per second"""
        return self._get_iio_attr(
            self._tx_channel_names[0], "sampling_frequency", True, self._txdac
        )



# Copyright (C) 2021-2024 Analog Devices, Inc.
#
# SPDX short identifier: ADIBSD

from typing import Dict, List

from adi.context_manager import context_manager
from adi.rx_tx import rx
from adi.sync_start import sync_start


class ad9083(sync_start, rx, context_manager):
    """AD9083 High-Speed Multi-channel ADC"""

    _complex_data = False
    _rx_channel_names: List[str] = []

    _device_name = ""

    def __init__(self, uri=""):

        context_manager.__init__(self, uri, self._device_name)

        self._rxadc = self._ctx.find_device("axi-ad9083-rx-hpc")
        if not self._rxadc:
            raise Exception("Cannot find device axi-ad9083-rx-hpc")

        self._rx_channel_names = []
        for ch in self._rxadc.channels:
            if ch.scan_element and not ch.output:
                self._rx_channel_names.append(ch._id)

        for name in self._rx_channel_names:
            if "_i" or "_q" in name:
                self._complex_data = True

        rx.__init__(self)

    @property
    def rx_sample_rate(self):
        """rx_sampling_frequency: Sample rate after decimation"""
        return self._get_iio_attr(
            "voltage0_i", "sampling_frequency", False, self._rxadc
        )

    @property
    def nco0_frequency(self):
        """nco0_frequency: Get/Set NCO0 frequency"""
        return self._get_iio_attr("altvoltage0", "frequency", True, self._rxadc)

    @nco0_frequency.setter
    def nco0_frequency(self, value):
        self._set_iio_attr_int(
            "altvoltage0", "frequency", True, int(value), self._rxadc
        )

    @property
    def nco1_frequency(self):
        """nco0_frequency: Get/Set NCO1 frequency"""
        return self._get_iio_attr("altvoltage1", "frequency", True, self._rxadc)

    @nco1_frequency.setter
    def nco1_frequency(self, value):
        self._set_iio_attr_int(
            "altvoltage1", "frequency", True, int(value), self._rxadc
        )

    @property
    def nco2_frequency(self):
        """nco0_frequency: Get/Set NCO2 frequency"""
        return self._get_iio_attr("altvoltage2", "frequency", True, self._rxadc)

    @nco2_frequency.setter
    def nco2_frequency(self, value):
        self._set_iio_attr_int(
            "altvoltage2", "frequency", True, int(value), self._rxadc
        )

    def reg_read(self, reg):
        """Direct Register Access via debugfs"""
        self._set_iio_debug_attr_str("direct_reg_access", reg, self._rxadc)
        return self._get_iio_debug_attr_str("direct_reg_access", self._rxadc)

    def reg_write(self, reg, value):
        """Direct Register Access via debugfs"""
        self._set_iio_debug_attr_str("direct_reg_access", f"{reg} {value}", self._rxadc)



# Copyright (C) 2023-2024 Analog Devices, Inc.
#
# SPDX short identifier: ADIBSD

import adi


class cn0575:
    """ CN0575 class, exposing onboard temperature sensor, pushbutton,
    and LED. Also reads the platform CPU's temperature, which under
    most operating conditions should be higher than the onboard sensor.
    """

    def __init__(self, uri=""):

        self.gpios = adi.one_bit_adc_dac(uri)
        self.adt75 = adi.lm75(uri)  # ADI version of LM75
        self.gpios.gpio_ext_led = 0  # turn off LED

    @property
    def button(self):
        """Read button state."""
        return self.gpios.gpio_ext_btn

    @property
    def led(self):
        """Read LED state."""
        return self.gpios.gpio_ext_led

    @led.setter
    def led(self, value):
        """Set LED state"""
        self.gpios.gpio_ext_led = value



# Copyright (C) 2022-2024 Analog Devices, Inc.
#
# SPDX short identifier: ADIBSD

from adi.attribute import attribute
from adi.context_manager import context_manager


class lm75(context_manager, attribute):

    """ LM75 Temperature Sensor

    Parameters
    ----------
    uri: type=string
        Context URI. Default: Empty (auto-scan)
    device_index: type=integer
        Device index in contexts with multiple LM75 compatible devices. Default: 0
    returns:
        LM75 compatible device
    """

    _device_name = ""

    def __init__(self, uri="", device_index=0):

        context_manager.__init__(self, uri, self._device_name)

        compatible_parts = ["lm75", "adt75"]

        self._ctrl = None
        index = 0
        # Select the device_index-th device from the lm75 family as working device.
        for device in self._ctx.devices:
            if device.name in compatible_parts:
                if index == device_index:
                    self._ctrl = device
                    break
                else:
                    index += 1

    @property
    def update_interval(self):
        """Update Interval"""
        return self._get_iio_dev_attr("update_interval")

    def to_degrees(self, value):
        """Convert raw to degrees Celsius"""
        return value / 1000.0

    def to_millidegrees(self, value):
        """Convert degrees Celsius to millidegrees"""
        return int(value * 1000.0)

    @property
    def input(self):
        """LM75 temperature input value"""
        return self._get_iio_attr("temp1", "input", False)

    @property
    def max(self):
        """LM75 temperature max value"""
        return self._get_iio_attr("temp1", "max", False)

    @max.setter
    def max(self, value):
        """LM75 temperature max value"""
        return self._set_iio_attr("temp1", "max", False, value)

    @property
    def max_hyst(self):
        """LM75 max_hyst value"""
        return self._get_iio_attr("temp1", "max_hyst", False)

    @max_hyst.setter
    def max_hyst(self, value):
        """LM75 max_hyst value"""
        return self._set_iio_attr("temp1", "max_hyst", False, value)

    def __call__(self):
        """Utility function, returns deg. C"""
        return self.input / 1000



# Copyright (C) 2023-2024 Analog Devices, Inc.
#
# SPDX short identifier: ADIBSD
"""Compatibility module for libiio v1.X."""

from typing import List, Union

import iio

import numpy as np


def _is_libiio_v1() -> bool:
    """Check is we are using >= v1.X."""
    v = iio.version
    return v[0] >= 1


class compat_libiio_v1_rx:
    """Compatibility class for libiio v1.X RX."""

    _rx_buffer_mask = None
    _rx_stream = None
    _rx_buffer_num_blocks = 4

    def _rx_init_channels(self):
        if not self._rx_buffer_mask:
            self._rx_buffer_mask = iio.ChannelsMask(self._rxadc)

        channels = []
        if self._complex_data:
            for m in self.rx_enabled_channels:
                v = self._rxadc.find_channel(self._rx_channel_names[m * 2])
                channels.append(v)
                v = self._rxadc.find_channel(self._rx_channel_names[m * 2 + 1])
                channels.append(v)
        else:
            for m in self.rx_enabled_channels:
                v = self._rxadc.find_channel(self._rx_channel_names[m])
                channels.append(v)

        self._rx_buffer_mask.channels = channels

        self._rxbuf = iio.Buffer(self._rxadc, self._rx_buffer_mask)
        self._rx_stream = iio.Stream(
            buffer=self._rxbuf,
            nb_blocks=self._rx_buffer_num_blocks,
            samples_count=self.rx_buffer_size,
        )

    def _rx_buffered_data(self):
        if not self._rx_stream:
            self._rx_init_channels()

        block = next(self._rx_stream)

        data_channel_interleaved = []
        for chan in self._rx_buffer_mask.channels:
            bytearray_data = chan.read(block)
            # create format strings
            df = chan.data_format
            fmt = ("i" if df.is_signed is True else "u") + str(df.length // 8)
            data_channel_interleaved.append(np.frombuffer(bytearray_data, dtype=fmt))

        return data_channel_interleaved


class compat_libiio_v1_tx:
    """Compatibility class for libiio v1.X TX."""

    _tx_buffer_mask = None
    _tx_stream = None
    _tx_buffer_num_blocks = 4
    _tx_block = None

    def _tx_init_channels(self):
        if not self._tx_buffer_mask:
            self._tx_buffer_mask = iio.ChannelsMask(self._txdac)

        channels = []
        if self._complex_data:
            for m in self.tx_enabled_channels:
                v = self._txdac.find_channel(self._tx_channel_names[m * 2], True)
                channels.append(v)
                v = self._txdac.find_channel(self._tx_channel_names[m * 2 + 1], True)
                channels.append(v)
        else:
            for m in self.tx_enabled_channels:
                v = self._txdac.find_channel(self._tx_channel_names[m], True)
                channels.append(v)

        self._tx_buffer_mask.channels = channels

        self._txbuf = iio.Buffer(self._txdac, self._tx_buffer_mask)
        if not self._tx_cyclic_buffer:
            self._tx_stream = iio.Stream(
                buffer=self._txbuf,
                nb_blocks=self._tx_buffer_num_blocks,
                samples_count=self._tx_buffer_size,
            )
        else:
            self._tx_block = iio.Block(self._txbuf, self._tx_buffer_size)

    def _tx_buffer_push(self, data):
        """Push data to TX buffer.

        data: bytearray
        """
        if self._tx_cyclic_buffer:
            self._tx_block.write(data)
            self._tx_block.enqueue(None, self._tx_cyclic_buffer)
            self._txbuf.enabled = True
        else:
            block = next(self._tx_stream)
            block.write(data)


class compat_libiio_v0_rx:
    """Compatibility class for libiio v0.X RX."""

    def _rx_init_channels(self):
        for m in self._rx_channel_names:
            v = self._rxadc.find_channel(m)
            if not v:
                raise Exception(f"Channel {m} not found")
            v.enabled = False

        if self._complex_data:
            for m in self.rx_enabled_channels:
                v = self._rxadc.find_channel(self._rx_channel_names[m * 2])
                v.enabled = True
                v = self._rxadc.find_channel(self._rx_channel_names[m * 2 + 1])
                v.enabled = True
        else:
            for m in self.rx_enabled_channels:
                v = self._rxadc.find_channel(self._rx_channel_names[m])
                v.enabled = True
        self._rxbuf = iio.Buffer(self._rxadc, self._rx_buffer_size, False)

    def _rx_buffered_data(self) -> Union[List[np.ndarray], np.ndarray]:
        """_rx_buffered_data: Read data from RX buffer

        Returns:
            List of numpy arrays containing the data from the RX buffer that are
            channel interleaved
        """
        if not self._rxbuf:
            self._rx_init_channels()
        self._rxbuf.refill()

        data_channel_interleaved = []
        ecn = []
        if self._complex_data:
            for m in self.rx_enabled_channels:
                ecn.extend(
                    (self._rx_channel_names[m * 2], self._rx_channel_names[m * 2 + 1])
                )
        else:
            ecn = [self._rx_channel_names[m] for m in self.rx_enabled_channels]

        for name in ecn:
            chan = self._rxadc.find_channel(name)
            bytearray_data = chan.read(self._rxbuf)  # Do local type conversion
            # create format strings
            df = chan.data_format
            fmt = ("i" if df.is_signed is True else "u") + str(df.length // 8)
            data_channel_interleaved.append(np.frombuffer(bytearray_data, dtype=fmt))

        return data_channel_interleaved


class compat_libiio_v0_tx:
    """Compatibility class for libiio v0.X TX."""

    def _tx_init_channels(self):
        if self._complex_data:
            for m in self.tx_enabled_channels:
                v = self._txdac.find_channel(self._tx_channel_names[m * 2], True)
                v.enabled = True
                v = self._txdac.find_channel(self._tx_channel_names[m * 2 + 1], True)
                v.enabled = True
        else:
            for m in self.tx_enabled_channels:
                v = self._txdac.find_channel(self._tx_channel_names[m], True)
                v.enabled = True
        self._txbuf = iio.Buffer(
            self._txdac, self._tx_buffer_size, self._tx_cyclic_buffer
        )

    def _tx_buffer_push(self, data):
        self._txbuf.write(bytearray(data))
        self._txbuf.push()



# Copyright (C) 2020-2024 Analog Devices, Inc.
#
# SPDX short identifier: ADIBSD

import numpy as np
from adi.context_manager import context_manager
from adi.rx_tx import rx


class adis16507(rx, context_manager):
    """ ADIS16507 Precision, Miniature MEMS IMU """

    _complex_data = False
    _rx_channel_names = [
        "anglvel_x",
        "anglvel_y",
        "anglvel_z",
        "accel_x",
        "accel_y",
        "accel_z",
    ]
    _device_name = ""
    _rx_data_si_type = float

    def __init__(
        self, uri="", imu_dev_name="adis16507-3", trigger_name="adis16507-3-dev0"
    ):

        context_manager.__init__(self, uri, self._device_name)

        self._ctrl = self._ctx.find_device(imu_dev_name)
        self._rxadc = self._ctx.find_device(imu_dev_name)
        # Set default trigger
        self._trigger = self._ctx.find_device(trigger_name)
        self._rxadc._set_trigger(self._trigger)
        rx.__init__(self)
        self.rx_buffer_size = 16  # Make default buffer smaller

    @property
    def sample_rate(self):
        """sample_rate: Sample rate in samples per second"""
        return self._get_iio_dev_attr("sampling_frequency")

    @sample_rate.setter
    def sample_rate(self, value):
        self._set_iio_dev_attr_str("sampling_frequency", value)

    @property
    def filter_low_pass_3db_frequency(self):
        """filter_low_pass_3db_frequency: Bandwidth for the accelerometer and
        gyroscope channels"""
        return self._get_iio_dev_attr("filter_low_pass_3db_frequency")

    @filter_low_pass_3db_frequency.setter
    def filter_low_pass_3db_frequency(self, value):
        self._set_iio_dev_attr_str("filter_low_pass_3db_frequency", value)

    @property
    def current_timestamp_clock(self):
        """current_timestamp_clock: Source clock for timestamps"""
        return self._get_iio_dev_attr("current_timestamp_clock")

    @current_timestamp_clock.setter
    def current_timestamp_clock(self, value):
        self._set_iio_dev_attr_str("current_timestamp_clock", value)



# Copyright (C) 2021-2024 Analog Devices, Inc.
#
# SPDX short identifier: ADIBSD

from adi.attribute import attribute
from adi.context_manager import context_manager


class adrf5720(attribute, context_manager):
    """ ADRF5720 Digital Attenuator """

    _complex_data = False
    channel = "voltage0"
    _device_name = ""

    def __init__(self, uri="", device_name=""):
        context_manager.__init__(self, uri, self._device_name)

        compatible_parts = [
            "adrf5720",
            "adrf5730",
            "adrf5731",
        ]

        self._ctrl = None

        for device in self._ctx.devices:
            if device.name in [device_name] + compatible_parts:
                self._ctrl = device
                break

        # Raise an exception if the device isn't found
        if not self._ctrl:
            raise Exception("ADRF5720 device not found")

    @property
    def attenuation(self):
        """Sets attenuation of the ADRF5720"""
        return abs(self._get_iio_attr("voltage0", "hardwaregain", True, self._ctrl))

    @attenuation.setter
    def attenuation(self, value):
        self._set_iio_attr(
            "voltage0", "hardwaregain", True, -1 * abs(value), self._ctrl
        )



# Copyright (C) 2023-2024 Analog Devices, Inc.
#
# SPDX short identifier: ADIBSD


from decimal import Decimal

from adi.attribute import attribute
from adi.context_manager import context_manager
from adi.rx_tx import tx


class ad579x(tx, context_manager):
    """ AD579x DAC """

    _complex_data = False
    channel = []  # type: ignore
    _device_name = ""

    def __init__(self, uri="", device_name=""):
        """Constructor for AD579x class."""
        context_manager.__init__(self, uri, self._device_name)

        compatible_parts = [
            "ad5780",
            "ad5781",
            "ad5790",
            "ad5791",
            "ad5760",
        ]

        self._ctrl = None

        if not device_name:
            device_name = compatible_parts[0]
        else:
            if device_name not in compatible_parts:
                raise Exception(
                    f"Not a compatible device: {device_name}. Supported device names "
                    f"are: {','.join(compatible_parts)}"
                )

        # Select the device matching device_name as working device
        for device in self._ctx.devices:
            if device.name == device_name:
                self._ctrl = device
                self._txdac = device
                break

        if not self._ctrl:
            raise Exception("Error in selecting matching device")

        if not self._txdac:
            raise Exception("Error in selecting matching device")

        self.output_bits = []
        self.channel = []
        self._tx_channel_names = []
        for ch in self._ctrl.channels:
            name = ch.id
            self.output_bits.append(ch.data_format.bits)
            self._tx_channel_names.append(name)
            self.channel.append(self._channel(self._ctrl, name))

        tx.__init__(self)

    @property
    def powerdown_mode(self):
        """Ad579x powerdown_mode config"""
        return self._get_iio_dev_attr_str("powerdown_mode")

    @powerdown_mode.setter
    def powerdown_mode(self, value):
        self._set_iio_dev_attr_str("powerdown_mode", value)

    @property
    def powerdown_mode_available(self):
        """AD579x powedown mode available"""
        return self._get_iio_dev_attr_str("powerdown_mode_available")

    @property
    def sampling_frequency(self):
        """AD579x sampling frequency config"""
        return self._get_iio_dev_attr_str("sampling_frequency")

    @sampling_frequency.setter
    def sampling_frequency(self, value):
        self._set_iio_dev_attr_str("sampling_frequency", value)

    class _channel(attribute):
        """AD579x channel"""

        def __init__(self, ctrl, channel_name):
            self.name = channel_name
            self._ctrl = ctrl

        @property
        def raw(self):
            """AD579x channel raw value"""
            return self._get_iio_attr(self.name, "raw", True)

        @raw.setter
        def raw(self, value):
            self._set_iio_attr(self.name, "raw", True, str(int(value)))

        @property
        def offset(self):
            """AD579x channel offset"""
            return self._get_iio_attr(self.name, "offset", True)

        @offset.setter
        def offset(self, value):
            self._set_iio_attr(self.name, "offset", True, str(Decimal(value).real))

        @property
        def scale(self):
            """AD579x channel scale"""
            return self._get_iio_attr(self.name, "scale", True)

        @scale.setter
        def scale(self, value):
            self._set_iio_attr(self.name, "scale", True, str(Decimal(value).real))

        @property
        def powerdown(self):
            """AD579x powerdown config"""
            return self._get_iio_attr_str(self.name, "powerdown", True)

        @powerdown.setter
        def powerdown(self, value):
            self._set_iio_attr(self.name, "powerdown", True, value)

        @property
        def powerdown_available(self):
            """AD579x powedown available"""
            return self._get_iio_attr_str(self.name, "powerdown_available", True)



# Copyright (C) 2022-2024 Analog Devices, Inc.
#
# SPDX short identifier: ADIBSD

from adi.attribute import attribute
from adi.context_manager import context_manager


class adt7420(attribute, context_manager):

    _device_name = "adt7420"

    def __init__(self, uri=""):

        context_manager.__init__(self, uri, self._device_name)

        self._ctrl = self._ctx.find_device(self._device_name)

        if not self._ctrl:
            raise Exception("ADT7420 device not found")

        self.temp = self._channel(self._ctrl, "temp")

    class _channel(attribute):
        """ADT7420 Channel"""

        def __init__(self, ctrl, channel_name):
            self._ctrl = ctrl
            self.name = channel_name

        @property
        def temp_val(self):
            """ ADT7420 Channel Temperature Value """
            return self._get_iio_attr(self.name, "temp", False)

        @property
        def temp_max(self):
            """ ADT7420 Channel Max Temperature """
            return self._get_iio_attr(self.name, "temp_max", False)

        @temp_max.setter
        def temp_max(self, value):
            """ ADT7420 Channel Max Temperature """
            return self._set_iio_attr(self.name, "temp_max", False, value)

        @property
        def temp_min(self):
            """ ADT7420 Channel Min Temperature """
            return self._get_iio_attr(self.name, "temp_min", False)

        @temp_min.setter
        def temp_min(self, value):
            """ ADT7420 Channel Min Temperature """
            return self._set_iio_attr(self.name, "temp_min", False, value)

        @property
        def temp_crit(self):
            """ ADT7420 Channel Critical Temperature """
            return self._get_iio_attr(self.name, "temp_crit", False)

        @temp_crit.setter
        def temp_crit(self, value):
            """ ADT7420 Channel Critical Temperature """
            return self._set_iio_attr(self.name, "temp_crit", False, value)

        @property
        def temp_hyst(self):
            """ ADT7420 Channel Hysteresis Temperature """
            return self._get_iio_attr(self.name, "temp_hyst", False)

        @temp_hyst.setter
        def temp_hyst(self, value):
            """ ADT7420 Channel Hysteresis Temperature """
            return self._set_iio_attr(self.name, "temp_hyst", False, value)



# Copyright (C) 2023-2024 Analog Devices, Inc.
#
# SPDX short identifier: ADIBSD
# Author: Ivan Gil Mercano <ivangil.mercano@analog.com>

import numpy as np
from adi.ad5940 import ad5940
from adi.adg2128 import adg2128
from adi.context_manager import context_manager


class cn0565(ad5940, adg2128, context_manager):

    """The CN0565 class inherits features from both the AD5940 (providing high
    precision in impedance and electrochemical frontend) and the ADG2128
    (enabling arbitrary assignment of force and sense electrodes). '
    These combined functionalities are utilized for Electrical
    Impedance Tomography.

    parameters:
        uri: type=string
            URI of the platform
    """

    _device_name = "cn0565"

    def __init__(self, uri=""):
        context_manager.__init__(self, uri, self._device_name)
        ad5940.__init__(self)
        adg2128.__init__(self)
        self._switch_sequence = None
        self.add(0x71)
        self.add(0x70)
        self._electrode_count = 8
        self._force_distance = 1
        self._sense_distance = 1
        self.excitation_frequency = 10000

    @property
    def electrode_count(self):
        """electrode_count: Number of electrodes"""
        return self._electrode_count

    @electrode_count.setter
    def electrode_count(self, value):
        self._electrode_count = value

    @property
    def force_distance(self):
        """force_distance: Number of electrodes between forcing electrodes. 1 means they are adjacent"""
        return self._force_distance

    @force_distance.setter
    def force_distance(self, value):
        self._force_distance = value

    @property
    def sense_distance(self):
        """sense_distance: Number of electrodes between sensing electrodes. 1 means they are adjacent"""
        return self._sense_distance

    @sense_distance.setter
    def sense_distance(self, value):
        self._sense_distance = value

    @property
    def switch_sequence(self):
        """switch_sequence: type=np.array
            Sequence of combinations of forcing electrodes and sensing electrodes in the form of
            f+, s+, s-, s+
        """
        seq = 0
        ret = []
        for i in range(self.electrode_count):
            f_plus = i
            f_minus = (i + self.force_distance) % self.electrode_count
            for j in range(self.electrode_count):
                s_plus = j % self.electrode_count
                if s_plus == f_plus or s_plus == f_minus:
                    continue
                s_minus = (s_plus + self.sense_distance) % self.electrode_count
                if s_minus == f_plus or s_minus == f_minus:
                    continue
                ret.append((f_plus, s_plus, s_minus, f_minus))
                seq += 1

        return np.array(ret)

    @property
    def all_voltages(self):
        """all_voltages: type=np.array
            Voltage readings from different electrode combinations
        """
        if self._switch_sequence is None:
            self._switch_sequence = self.switch_sequence

        self.impedance_mode = False
        ret = []
        for seq in self._switch_sequence:
            # reset cross point switch
            self.gpio1_toggle = True

            # set new cross point switch configuration from pregenerated sequence
            self._xline[seq[0]][0] = True
            self._xline[seq[1]][1] = True
            self._xline[seq[2]][2] = True
            self._xline[seq[3]][3] = True

            # read impedance
            s = self.channel["voltage0"].raw
            ret.append([s.real, s.imag])

        return np.array(ret).reshape(len(ret), 2)

    @property
    def electrode_count_available(self):
        """electrode_count_available: type=np.array
            Supported Electrode Counts
        """
        return np.array([8, 16, 32])



# Copyright (C) 2024 Analog Devices, Inc.
#
# SPDX short identifier: ADIBSD

from decimal import Decimal

from adi.attribute import attribute
from adi.context_manager import context_manager
from adi.rx_tx import tx


class ad3552r(tx, context_manager):
    """ AD3552R DAC """

    _complex_data = False
    _device_name = "AD3552R"

    def __init__(self, uri="", device_name=""):
        """ Constructor for AD3552R driver class """

        context_manager.__init__(self, uri, self._device_name)

        compatible_parts = [
            "ad3552r",
            "ad3542r",
        ]

        self._ctrl = None
        self._txdac = None

        if not device_name:
            device_name = compatible_parts[0]
        else:
            if device_name not in compatible_parts:
                raise Exception(
                    f"Not a compatible device: {device_name}. Supported device names "
                    f"are: {','.join(compatible_parts)}"
                )

        # Select the device matching device_name as working device
        for device in self._ctx.devices:
            if device.name == device_name:
                self._ctrl = device
                self._txdac = device
                break

        if not self._ctrl:
            raise Exception("Error in selecting matching device")

        if not self._txdac:
            raise Exception("Error in selecting matching device")

        self.output_bits = []
        self.channel = []
        self._tx_channel_names = []
        for ch in self._ctrl.channels:
            name = ch._id
            output = ch._output
            self.output_bits.append(ch.data_format.bits)
            self._tx_channel_names.append(name)
            self.channel.append(self._channel(self._ctrl, name, output))
            if output is True:
                setattr(self, name, self._channel(self._ctrl, name, output))

        tx.__init__(self)

    class _channel(attribute):
        """AD3552R channel"""

        def __init__(self, ctrl, channel_name, output):
            self.name = channel_name
            self._ctrl = ctrl
            self._output = output

        @property
        def raw(self):
            """Get channel raw value
                DAC code in the range 0-65535"""
            return self._get_iio_attr(self.name, "raw", True)

        @raw.setter
        def raw(self, value):
            """Set channel raw value"""
            self._set_iio_attr(self.name, "raw", True, str(int(value)))

        @property
        def offset(self):
            """Get channel offset"""
            return self._get_iio_attr_str(self.name, "offset", True)

        @offset.setter
        def offset(self, value):
            """Set channel offset"""
            self._set_iio_attr(self.name, "offset", True, str(Decimal(value).real))

        @property
        def scale(self):
            """Get channel scale"""
            return float(self._get_iio_attr_str(self.name, "scale", True))

        @scale.setter
        def scale(self, value):
            """Set channel scale"""
            self._set_iio_attr(self.name, "scale", True, str(Decimal(value).real))



# Copyright (C) 2022-2024 Analog Devices, Inc.
#
# SPDX short identifier: ADIBSD
from collections import OrderedDict

import numpy as np
from adi.attribute import attribute
from adi.context_manager import context_manager
from adi.rx_tx import rx


class ltc2499(rx, context_manager):

    channel: OrderedDict = None
    _device_name = "ltc2499"
    _rx_unbuffered_data = True
    _rx_data_si_type = float

    def __init__(self, uri=""):
        context_manager.__init__(self, uri, self._device_name)
        self._ctrl = self._ctx.find_device("ltc2499")

        if not self._ctrl:
            raise Exception("No device found")

        _channels = []
        self._rx_channel_names = []
        for ch in self._ctrl.channels:
            self._rx_channel_names.append(ch.id)
            _channels.append((ch.id, self._channel(self._ctrl, ch.id)))
        self.channel = OrderedDict(_channels)

        rx.__init__(self)

    class _channel(attribute):
        def __init__(self, ctrl, channel_name):
            self._ctrl = ctrl
            self.name = channel_name

        @property
        def raw(self):
            return self._get_iio_attr(self.name, "raw", False)

        @property
        def scale(self):
            return float(self._get_iio_attr_str(self.name, "scale", False))

        @property
        def value(self):
            return self.raw * self.scale



# Copyright (C) 2024 Analog Devices, Inc.
#
# SPDX short identifier: ADIBSD

from decimal import Decimal

from adi.attribute import attribute
from adi.context_manager import context_manager


class ltc2672(context_manager, attribute):
    """LTC2672 DAC"""

    _complex_data = False
    _device_name = ""

    def __init__(self, uri="", device_name=""):
        """Constructor for LTC2672 class"""
        context_manager.__init__(self, uri, self._device_name)

        compatible_parts = ["ltc2672-16", "ltc2672-12"]

        self._ctrl = None
        self.channel = []

        if not device_name:
            device_name = compatible_parts[0]
        else:
            if device_name not in compatible_parts:
                raise Exception(
                    f"Not a compatible device: {device_name}. Supported device names"
                    f"are: {','.join(compatible_parts)}"
                )

        # Select the device matching device_name as working device
        for device in self._ctx.devices:
            if device.name == device_name:
                self._ctrl = device
                break

        if not self._ctrl:
            raise Exception("Error in selecting matching device")

        self.output_bits = []
        for ch in self._ctrl.channels:
            name = ch.id
            self.output_bits.append(ch.data_format.bits)
            self.channel.append(self._channel(self._ctrl, name))

    @property
    def all_chns_span(self):
        """Get all channels span in mA"""
        return self._get_iio_dev_attr_str("all_chns_span")

    @property
    def all_chns_span_avail(self):
        """Get list of span options in mA"""
        return self._get_iio_dev_attr_str("all_chns_span_available")

    @all_chns_span.setter
    def all_chns_span(self, value):
        """Set all channels span"""
        if value in self.all_chns_span_avail:
            self._set_iio_dev_attr_str("all_chns_span", value)
        else:
            raise ValueError(
                "Error: span setting not supported \nUse one of: "
                "str(self.all_chns_span_avail)"
            )

    @property
    def all_chns_raw(self):
        """Get raw value"""
        return self._get_iio_dev_attr_str("all_chns_raw")

    @all_chns_raw.setter
    def all_chns_raw(self, value):
        """Set raw value"""
        self._set_iio_dev_attr_str("all_chns_raw", value)

    @property
    def all_chns_current(self):
        """Get current value in mA"""
        return self._get_iio_dev_attr_str("all_chns_current")

    @all_chns_current.setter
    def all_chns_current(self, value):
        """Set current value in mA"""
        self._set_iio_dev_attr_str("all_chns_current", value)

    @property
    def all_chns_powerdown(self):
        """Get powerdown value"""
        return self._get_iio_dev_attr_str("all_chns_powerdown")

    @property
    def all_chns_powerdown_avail(self):
        """Get powerdown options"""
        return self._get_iio_dev_attr_str("all_chns_powerdown_available")

    @all_chns_powerdown.setter
    def all_chns_powerdown(self, value):
        """Set all channels powerdown"""
        if value in self.all_chns_powerdown_avail:
            self._set_iio_dev_attr_str("all_chns_powerdown", value)
        else:
            raise ValueError(
                "Error: powerdown option not supported \nUse one of: "
                "str(self.all_chns_powerdown_avail)"
            )

    @property
    def mux(self):
        """Get mux setting value"""
        return self._get_iio_dev_attr_str("mux")

    @property
    def mux_avail(self):
        """Get mux setting options"""
        return self._get_iio_dev_attr_str("mux_available")

    @mux.setter
    def mux(self, value):
        """Set mux option"""
        if value in self.mux_avail:
            self._set_iio_dev_attr_str("mux", value)
        else:
            raise ValueError(
                "Error: mux setting not supported \nUse one of: " "str(self.mux_avail)"
            )

    @property
    def fault_detect(self):
        """Get fault condition if any"""
        return self._get_iio_dev_attr_str("fault_detect")

    @property
    def fault_detect_avail(self):
        """Get fault detect options"""
        return self._get_iio_dev_attr_str("fault_detect_available")

    class _channel(attribute):
        """LTC2672 channel"""

        def __init__(self, ctrl, channel_name):
            self.name = channel_name
            self._ctrl = ctrl

        @property
        def raw(self):
            """Get channel raw value"""
            return self._get_iio_attr(self.name, "raw", True)

        @raw.setter
        def raw(self, value):
            """Set channel raw value"""
            self._set_iio_attr(self.name, "raw", True, str(int(value)))

        @property
        def scale(self):
            """Get channel scale"""
            return self._get_iio_attr(self.name, "scale", True)

        @property
        def offset(self):
            """get channel offset"""
            return self._get_iio_attr(self.name, "offset", True)

        @property
        def current(self):
            """Get channel current value in mA"""
            return self._get_iio_attr(self.name, "current", True)

        @current.setter
        def current(self, value):
            """Set channel current value in mA"""
            self._set_iio_attr(self.name, "current", True, str(Decimal(value)))

        @property
        def span(self):
            """Get channel span value"""
            return self._get_iio_attr_str(self.name, "span", True)

        @property
        def span_avail(self):
            """Get channel span options"""
            return self._get_iio_attr_str(self.name, "span_available", True)

        @span.setter
        def span(self, value):
            """Set channel span setting"""
            if value in self.span_avail:
                self._set_iio_attr(self.name, "span", True, value)
            else:
                raise ValueError(
                    "Error: span setting not supported \nUse one of: "
                    "str(self.span_avail)"
                )

        @property
        def powerdown(self):
            """Get channel powerdown"""
            return self._get_iio_attr_str(self.name, "powerdown", True)

        @property
        def powerdown_avail(self):
            """Get channel powerdown options"""
            return self._get_iio_attr_str(self.name, "powerdown_available", True)

        @powerdown.setter
        def powerdown(self, value):
            """Set channel powerdown setting"""
            if value in self.powerdown_avail:
                self._set_iio_attr(self.name, "powerdown", True, value)
            else:
                raise ValueError(
                    "Error: powerdown setting not supported \nUse one of: "
                    "str(self.powerdown_avail)"
                )



# Copyright (C) 2019-2024 Analog Devices, Inc.
#
# SPDX short identifier: ADIBSD

import numpy as np
from adi.context_manager import context_manager
from adi.rx_tx import rx
from adi.sync_start import sync_start


class ad9094(sync_start, rx, context_manager):
    """ AD9094 Quad ADC """

    _complex_data = False
    _rx_channel_names = ["voltage0", "voltage1", "voltage2", "voltage3", "voltage4"]
    _device_name = ""

    def __init__(self, uri=""):
        context_manager.__init__(self, uri, self._device_name)
        self._rxadc = self._ctx.find_device("axi-ad9094-hpc")
        rx.__init__(self)



# Copyright (C) 2020-2024 Analog Devices, Inc.
#
# SPDX short identifier: ADIBSD

import time

from adi.cn0540 import cn0540, reset_buffer


class cn0532(cn0540):
    """CN0532: Custom board with ADXL1002 Low Noise, High Frequency +/-50g MEMS Accelerometer"""

    def __init__(self, uri=""):

        cn0540.__init__(self, uri=uri)

    @reset_buffer
    def calibrate(self):
        """Tune LTC2606 to make AD7768-1 ADC codes zero mean"""
        adc_chan = self._rxadc
        dac_chan = self._ltc2606
        adc_scale = float(self._get_iio_attr("voltage0", "scale", False, adc_chan))
        dac_scale = float(self._get_iio_attr("voltage0", "scale", True, dac_chan))

        for _ in range(20):
            raw = self._get_iio_attr("voltage0", "raw", False, adc_chan)
            adc_voltage = raw * adc_scale

            raw = self._get_iio_attr("voltage0", "raw", True, dac_chan)
            dac_voltage = (raw * dac_scale - adc_voltage) / dac_scale

            e = int(dac_voltage * dac_scale)

            if int(dac_voltage) > 2 ** 16 - 1:
                print(
                    "Warning: DAC voltage at upper limit, "
                    + f"calibration may not converge (Error: {e - (2**16 - 1)} codes).\n"
                    + "Make sure sensor is connected."
                )
                dac_voltage = 2 ** 16 - 1
            elif int(dac_voltage) < 0:
                print(
                    "Warning: DAC voltage at lower limit, "
                    + f"calibration may not converge (Error: {e} codes).\n"
                    + "Make sure sensor is connected."
                )
                dac_voltage = 0

            self._set_iio_attr_float(
                "voltage0", "raw", True, int(dac_voltage), dac_chan
            )
            time.sleep(0.01)
